# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import curses
from datetime import timedelta
# Page class definition
from .jtopgui import Page
# Graphics elements
from .lib.common import plot_name_info
# Menu GUI pages
from .jtopguimenu import strfdelta


class INFO(Page):

    def __init__(self, stdscr, jetson, refresh):
        super(INFO, self).__init__("INFO", stdscr, jetson, refresh)

    def info_variable(self, start, offset, name, value, spacing=18):
        self.stdscr.addstr(start, offset, name + ":")
        self.stdscr.addstr(start, offset + spacing, value, curses.A_BOLD)

    def draw(self, key, mouse):
        """
            Write all environment variables
        """
        # Screen size
        _, width, first = self.size_page()
        # Position information
        posx = 1
        start_pos = first + 2
        spacing = 18
        # Up time
        uptime_string = strfdelta(timedelta(seconds=self.jetson.uptime), "{days} days {hours}:{minutes}:{seconds}")
        self.stdscr.addstr(start_pos, posx, "- Up Time:", curses.A_BOLD)
        self.stdscr.addstr(start_pos, posx + spacing, uptime_string)
        start_pos += 1
        # Loop build information
        idx = 0
        # Board info
        self.stdscr.addstr(start_pos + idx, posx, "- Jetpack:", curses.A_BOLD)
        self.stdscr.addstr(start_pos + idx, posx + spacing, self.jetson.board["info"]["Jetpack"], curses.A_BOLD)
        self.stdscr.addstr(start_pos + idx + 1, posx, "- Board:", curses.A_BOLD)
        idx += 2
        # Load Board information
        board = self.jetson.board["board"]
        self.info_variable(start_pos + idx, posx + 2, "* Type", board["TYPE"])
        self.info_variable(start_pos + idx + 1, posx + 2, "* SOC Family", board["SOC"])
        self.info_variable(start_pos + idx + 1, posx + 33, "ID", board["CHIP_ID"], spacing=4)
        self.info_variable(start_pos + idx + 2, posx + 2, "* Module", board["MODULE"])
        self.info_variable(start_pos + idx + 2, posx + 33, "Board", board["BOARD"], spacing=7)
        self.info_variable(start_pos + idx + 3, posx + 2, "* Code Name", board["CODENAME"])
        self.info_variable(start_pos + idx + 4, posx + 2, "* Cuda ARCH", board["CUDA_ARCH_BIN"])
        self.info_variable(start_pos + idx + 5, posx + 2, "* Serial Number", board["SERIAL_NUMBER"])
        idx += 5
        if board["BOARDIDS"]:
            self.info_variable(start_pos + idx + 1, posx + 2, "* Board ids", board["BOARDIDS"])
            idx += 1
        # Libraries info
        library_line = start_pos + idx + 1
        self.stdscr.addstr(library_line, posx, "- Libraries:", curses.A_BOLD)
        idx += 1
        for name, info in sorted(self.jetson.board["libraries"].items()):
            if name == "OpenCV-Cuda":
                continue
            self.stdscr.addstr(start_pos + idx + 1, posx + 2, "* " + name + ":")
            self.stdscr.addstr(start_pos + idx + 1, posx + spacing, info, curses.A_BOLD)
            if name == "OpenCV":
                self.stdscr.addstr(start_pos + idx + 1, posx + 25, "compiled CUDA:")
                cuda = self.jetson.board["libraries"]["OpenCV-Cuda"]
                color = curses.color_pair(2) if cuda == "YES" else curses.color_pair(1)
                self.stdscr.addstr(start_pos + idx + 1, posx + 40, cuda, color | curses.A_BOLD)
            idx += 1
        # IP address and Hostname
        if self.jetson.local_interfaces:
            # Write hostname
            self.stdscr.addstr(library_line, width - 35, "- Hostname:", curses.A_BOLD)
            self.stdscr.addstr(library_line, width - 20, self.jetson.local_interfaces["hostname"], curses.A_BOLD)
            # Write all interfaces
            self.stdscr.addstr(library_line + 1, width - 35, "- Interfaces:", curses.A_BOLD)
            idx = 2
            for name, ip in self.jetson.local_interfaces["interfaces"].items():
                self.stdscr.addstr(library_line + idx, width - 33, "* " + name + ":")
                self.stdscr.addstr(library_line + idx, width - 20, ip, curses.A_BOLD)
                idx += 1
        # Author information
        plot_name_info(self.stdscr, start_pos - 1, width - 31, "Version", self.jetson.version)
        plot_name_info(self.stdscr, start_pos, width - 30, "Author", "Raffaello Bonghi")
        plot_name_info(self.stdscr, start_pos + 1, width - 30, "e-mail", "raffaello@rnext.it")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2020 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os

# Shell infromation
# os.environ['SHELL']


def jetpack_missing(repository, version):
    l4t = os.environ["JETSON_L4T"]
    # Title
    title = "Jetpack missing [L4T {l4t}]".format(l4t=l4t)
    # Template
    template = "jetpack-missing.md"
    # Body
    body = "Please update jetson-stats with new jetpack\n\n"
    body += "**Linux for Tegra**\n"
    body += " - L4T: " + l4t + "\n\n"
    body += "**Jetson-Stats**\n"
    body += " - Version: " + version + "\n"
    # Make url
    url = make_issue(repository, title, body=body, labels="missing", template=template)
    # message shell
    return hyperlink(url, title)


def board_missing(repository, version):
    board = os.environ["JETSON_BOARD"]
    # Title
    title = "Board missing {board}".format(board=board)
    # Template
    template = "board-missing.md"
    # Body
    body = "Please update jetson-stats with this board\n\n"
    body += "**Board**\n"
    body += " - Board(s): " + board + "\n"
    body += " - Boardis: " + os.environ["JETSON_BOARDIDS"] + "\n"
    body += " - SOC: " + os.environ["JETSON_SOC"] + "\n"
    body += " - ID: " + os.environ["JETSON_CHIP_ID"] + "\n"
    body += " - Code Name: " + os.environ["JETSON_CODENAME"] + "\n\n"
    body += "**Jetpack**\n"
    body += " - Jetpack: " + os.environ["JETSON_JETPACK"] + "\n"
    body += " - L4T: " + os.environ["JETSON_L4T"] + "\n\n"
    body += "**Jetson-Stats**\n"
    body += " - version: " + version + "\n"
    # Make url
    url = make_issue(repository, title, body=body, labels="missing", template=template)
    # message shell
    return hyperlink(url, title)


def hyperlink(url, text, hyperlink=True):
    # Reference:
    # 1. http://www.lihaoyi.com/post/BuildyourownCommandLinewithANSIescapecodes.html
    # 2. https://stackoverflow.com/questions/40419276/python-how-to-print-text-to-console-as-hyperlink
    # 3. https://purpleidea.com/blog/2018/06/29/hyperlinks-in-gnome-terminal/
    # 4. https://gist.github.com/egmontkob/eb114294efbcd5adb1944c9f3cb5feda
    # 5. https://stackoverflow.com/questions/44078888/clickable-html-links-in-python-3-6-shell
    # Check type of shell
    if hyperlink:
        return u"\u001b]8;;{url}\u001b\\{text}\u001b]8;;\u001b\\ (CTRL + Click to open this issue)".format(url=url, text=text)
    else:
        return "{text} ({url})".format(url=url, text=text)


def make_issue(repository, title, body="", labels="", template=""):
    # Reference
    # https://help.github.com/en/enterprise/2.17/user/github/managing-your-work-on-github/about-automation-for-issues-and-pull-requests-with-query-parameters
    # https://krypted.com/utilities/html-encoding-reference/
    title = title.replace(" ", "+").replace("[", "%5B").replace("]", "%5D")
    url = "{repository}/new?title={title}".format(repository=repository, title=title)
    if labels:
        url += "&labels=" + labels
    if template:
        url += "&template=" + template
    if body:
        body = body.replace(" ", "+").replace("#", "%23").replace("\n", "%0A").replace("*", "%2A")
        url += "&body=" + body
    return url
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import re
# Logging
import logging
# Launch command
import subprocess as sp
# Functions and decorators
from functools import wraps

# Create logger for jplotlib
logger = logging.getLogger(__name__)


def jetson_clocks_checks(func):
    """ Enable and disable Jetson Clocks after NVP model change """
    @wraps(func)
    def wrapped(self, level):
        # Disable the jetson_clocks (only if is it active) before change NVPmodel level
        if self.jetson_clocks is not None:
            old_status = self.jetson_clocks.start
            if old_status:
                self.jetson_clocks.start = False
        status = func(self, level)
        # Enable again the jetson_clocks status
        if self.jetson_clocks is not None:
            if old_status:
                self.jetson_clocks.start = True
        # Return status function
        return status
    return wrapped


class NVPmodel():
    """
        This controller read the status from your NVIDIA Jetson and you can control
        performance and status.
        It is available for NVIDIA Jetson TX2, Jetson AGX Xavier, Jetson Nano
        Boards reference:
        * TX2: https://www.jetsonhacks.com/2017/03/25/nvpmodel-nvidia-jetson-tx2-development-kit/
        * AGX Xavier: https://www.jetsonhacks.com/2018/10/07/nvpmodel-nvidia-jetson-agx-xavier-developer-kit/
        * Nano: https://www.jetsonhacks.com/2019/04/10/jetson-nano-use-more-power/
    """
    REGEXP = re.compile(r'POWER_MODEL: ID=(.+?) NAME=((.*))')
    REGPM = re.compile(r'NV Power Mode: ((.*))')

    class NVPmodelException(Exception):
        pass

    def __init__(self, jetson_clocks=None):
        self.jetson_clocks = jetson_clocks
        try:
            nvpmodel_p = sp.Popen(['nvpmodel', '-p', '--verbose'], stdout=sp.PIPE)
            out, _ = nvpmodel_p.communicate()
            # Log value
            logger.debug('nvqmodel status %s', out)
            # Decode lines
            lines = out.decode("utf-8")
            # Read all lines and extract modes
            self.board = []
            for line in lines.split("\n"):
                # Search configuration NVPmodel
                match = NVPmodel.REGEXP.search(line)
                # if match extract name and number
                if match:
                    mode = str(match.group(2))
                    pm = {"ID": int(match.group(1)), "Name": mode.replace("MODE_", "").replace("_", " "), "status": True}
                    self.board += [pm]
        except OSError:
            logger.info("This board does not have NVP Model")
            raise NVPmodel.NVPmodelException("NVPmodel does not exist for this board")
        except AttributeError:
            logger.info("Wrong open")
            raise NVPmodel.NVPmodelException("Wrong open")
        # Initialize mode and num
        self.update()
        # Initialize with first configuration number
        self.selected = self.num

    @property
    def modes(self):
        return self.board

    @jetson_clocks_checks
    def set(self, level):
        """ Set nvpmodel to a new status """
        try:
            self.selected = level
            # Set the new nvpmodel status
            sep_nvp = sp.Popen(['nvpmodel', '-m', str(level)], stdout=sp.PIPE, stderr=sp.PIPE, stdin=sp.PIPE)
            out, _ = sep_nvp.communicate()
            if "NVPM ERROR" in out.decode("utf-8"):
                self.board[level]["status"] = False
                return False
            else:
                self.board[level]["status"] = True
                return True
        except OSError:
            logger.info("NVP Model does not exist")
            return False
        except AttributeError:
            logger.info("Wrong open")
            return False

    def increase(self):
        if self.selected + 1 < len(self.modes):
            self.selected += 1
            return self.set(self.selected)
        else:
            return False

    def decrease(self):
        if self.selected - 1 >= 0:
            self.selected -= 1
            return self.set(self.selected)
        else:
            return False

    def update(self):
        """ Read nvpmodel to know the status of the board """
        try:
            nvpmodel_p = sp.Popen(['nvpmodel', '-q'], stdout=sp.PIPE, stderr=sp.PIPE)
            out, _ = nvpmodel_p.communicate()
            # Log value
            logger.debug('nvqmodel status %s', out)
            # Decode lines and split
            lines = out.decode("utf-8").split("\n")
            # Extract lines
            for idx, line in enumerate(lines):
                # Search configuration NVPmodel
                match = NVPmodel.REGPM.search(line)
                # if match extract name and number
                if match:
                    # Extract NV Power Mode
                    mode = str(match.group(1))
                    self.mode = mode.replace("MODE_", "").replace("_", " ")
                    # Extract number
                    self.num = int(lines[idx + 1])
                    break
        except OSError:
            logger.info("NVP Model does not exist")
        except AttributeError:
            logger.info("Wrong open")
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import curses
from curses.textpad import rectangle
import platform
# Page class definition
from .jtopgui import Page
from .lib.chart import Chart
from .lib.common import (check_curses,
                         plot_name_info,
                         label_freq)


class CPU(Page):

    def __init__(self, stdscr, jetson, refresh):
        super(CPU, self).__init__("CPU", stdscr, jetson, refresh)
        # List all CPU
        self.chart_cpus = []
        n_cpu = len(self.jetson.stats["CPU"])
        for idx in range(n_cpu):
            self.chart_cpus += [Chart(jetson,
                                      "CPU {idx}".format(idx=idx + 1),
                                      refresh,
                                      self.update_chart,
                                      color=curses.color_pair(4),
                                      color_chart=[curses.color_pair(10)])]

    def update_chart(self, jetson, name):
        n = int(name.split(" ")[1]) - 1
        cpu = jetson.stats["CPU"][n]
        status = cpu.get("status", "OFF") == "ON"
        return {
            'value': [cpu.get("val", 0)],
            'active': status
        }

    @check_curses
    def draw(self, key, mouse):
        """
            Draw a plot with GPU payload
        """
        n_cpu = sum([1 if cpu.get("status", "OFF") == "ON" else 0 for cpu in self.jetson.stats["CPU"]])
        # Screen size
        height, width, first = self.size_page()
        # Make all CPU charts
        counter = 0
        x_offset = width // 4
        # Draw info rectangle
        rectangle(self.stdscr, first + 1, 0, height - 2 + first, first + x_offset)
        # Draw title side area
        self.stdscr.addstr(first + 1, 1, " Platform ", curses.A_BOLD)
        # Platform information
        arch = platform.machine()
        plot_name_info(self.stdscr, first + 2, 2, "Arch", arch)
        release = platform.release()
        release = release[:x_offset - (first + 3) - 4]
        plot_name_info(self.stdscr, first + 3, 2, "Rel", release)
        # Architecture CPU cores
        architecture = self.jetson.architecture
        offset_table = 5
        for idx, cpu in enumerate(self.jetson.stats["CPU"]):
            status = cpu.get("status", "OFF")
            active = status == "ON"
            frq = label_freq(cpu)
            # Load governor if exist
            governor = cpu.get("governor", "")
            governor = governor[:x_offset - (first + 3) - 4]
            # Load model architecture
            model = ""
            if idx in architecture:
                cpu_arch = architecture[idx]
                model = cpu_arch.get("model name", "").split()[0]
                model = model[:x_offset - (first + 3) - 4]
            # Write label CPU name
            cpu_name = "CPU {n}:".format(n=idx + 1)
            # Draw info
            color = curses.color_pair(8) if active else curses.color_pair(7)
            self.stdscr.addstr(first + offset_table + idx * 2, 2, cpu_name, color)
            try:
                if active:
                    self.stdscr.addstr(first + offset_table + idx * 2, 2 + len(cpu_name) + 1, model, curses.A_NORMAL)
                    self.stdscr.addstr(first + offset_table + idx * 2 + 1, 2, frq, curses.A_NORMAL)
                    self.stdscr.addstr(first + offset_table + idx * 2 + 1, 2 + len(cpu_name) + 1, governor, curses.A_NORMAL)
                else:
                    self.stdscr.addstr(first + offset_table + idx * 2, 2 + len(cpu_name) + 1, status, curses.A_NORMAL)
            except curses.error:
                pass
        # Evaluate size single chart
        x_size = (width - x_offset - 6) // (n_cpu // 2)
        y_size = (height - 2 - first) // 2
        # Measure offset label
        offest_label = width - (2 + x_offset + x_size * (n_cpu // 2))
        # Plot all CPUs
        idxn = 0
        for idx, (cpu, data) in enumerate(zip(self.chart_cpus, self.jetson.stats["CPU"])):
            # status CPU
            status = data.get("status", "OFF") == "ON"
            if not status:
                continue
            y_label = idxn % (n_cpu // 2) == (n_cpu // 2) - 1
            # Select line
            line = 1 if idxn >= n_cpu // 2 else 0
            # Incrase counter
            counter = idxn - line * (n_cpu // 2)
            # Evaluate size chart
            add_size = offest_label if y_label else 0
            size_x = [x_offset + 2 + (counter * (x_size)), x_offset + x_size * (1 + counter) + add_size]
            size_y = [first + 1 + (line * (y_size)), first + y_size * (1 + line)]
            # Value and frequency
            if status:
                label_chart_cpu = "{percent: >2}%".format(percent=data['val']) if 'val' in data else ""
                cpu.draw(self.stdscr, size_x, size_y, label=label_chart_cpu, y_label=y_label)
            idxn += 1
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import socket
from service import JtopServer
from .core import import_os_variables
# Version match
VERSION_RE = re.compile(r""".*__version__ = ["'](.*?)['"]""", re.S)


def import_jetson_variables():
    JTOP_FOLDER, _ = os.path.split(__file__)
    return import_os_variables(JTOP_FOLDER + "/jetson_variables", "JETSON_")


def get_version():
    """
    Show the version of this package

    :return: Version number
    :rtype: string
    """
    # Load version package
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, "__init__.py")) as fp:
        VERSION = VERSION_RE.match(fp.read()).group(1)
    return VERSION

class jtop:
    class JtopException(Exception):
        """ Jtop general exception """
        pass

    def __init__(self, interval=500):
        # Open socket
        self.sock_ctrl = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)

    def open(self):
        try:
            self.sock_ctrl.sendto("start", JtopServer.PIPE_JTOP_CTRL)
        except socket.error as e:
            print(e)
            raise jtop.JtopException("jtop server not available")
        print("Open library")

    def close(self):
        try:
            self.sock_ctrl.sendto("stop", JtopServer.PIPE_JTOP_CTRL)
        except socket.error:
            raise jtop.JtopException("jtop server not available")
        print("Close library")

    def __enter__(self):
        """ Enter function for 'with' statement """
        self.open()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """ Exit function for 'with' statement """
        self.close()
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import abc
import curses
import signal
# Logging
import logging
# Timer
from datetime import datetime, timedelta
# Graphics elements
from .lib.common import (check_size,
                         check_curses,
                         set_xterm_title,
                         xterm_line)
# Create logger for jplotlib
logger = logging.getLogger(__name__)
# Initialization abstract class
# In according with: https://gist.github.com/alanjcastonguay/25e4db0edd3534ab732d6ff615ca9fc1
ABC = abc.ABCMeta('ABC', (object,), {})


class Page(ABC):

    def __init__(self, name, stdscr, jetson, refresh):
        self.name = name
        self.stdscr = stdscr
        self.jetson = jetson
        self.refresh = refresh

    def size_page(self):
        height, width = self.stdscr.getmaxyx()
        first = 0
        # Remove a line for sudo header
        if self.jetson.userid != 0:
            height -= 1
            first = 1
        return height, width, first

    @abc.abstractmethod
    @check_curses
    def draw(self, key, mouse):
        pass

    def keyboard(self, key):
        pass


class JTOPGUI:
    """
        The easiest way to use curses is to use a wrapper around a main function
        Essentially, what goes in the main function is the body of your program,
        The `stdscr' parameter passed to it is the curses screen generated by our
        wrapper.
    """
    COLORS = {"RED": 1, "GREEN": 2, "YELLOW": 3, "BLUE": 4, "MAGENTA": 5, "CYAN": 6}

    def __init__(self, stdscr, refresh, jetson, pages, init_page=0, start=True, loop=False, seconds=5):
        # Define pairing colors
        curses.init_pair(1, curses.COLOR_RED, curses.COLOR_BLACK)
        curses.init_pair(2, curses.COLOR_GREEN, curses.COLOR_BLACK)
        curses.init_pair(3, curses.COLOR_YELLOW, curses.COLOR_BLACK)
        curses.init_pair(4, curses.COLOR_BLUE, curses.COLOR_BLACK)
        curses.init_pair(5, curses.COLOR_MAGENTA, curses.COLOR_BLACK)
        curses.init_pair(6, curses.COLOR_CYAN, curses.COLOR_BLACK)
        # background
        curses.init_pair(7, curses.COLOR_WHITE, curses.COLOR_RED)
        curses.init_pair(8, curses.COLOR_WHITE, curses.COLOR_GREEN)
        curses.init_pair(9, curses.COLOR_WHITE, curses.COLOR_YELLOW)
        curses.init_pair(10, curses.COLOR_WHITE, curses.COLOR_BLUE)
        curses.init_pair(11, curses.COLOR_WHITE, curses.COLOR_MAGENTA)
        curses.init_pair(12, curses.COLOR_WHITE, curses.COLOR_CYAN)
        # Set curses reference, refresh and jetson controller
        self.stdscr = stdscr
        self.refresh = refresh
        self.jetson = jetson
        # Initialize all Object pages
        self.pages = [obj(stdscr, jetson, refresh) for obj in pages]
        # Set default page
        self.n_page = 0
        self.set(init_page)
        # Initialize keyboard status
        self.key = -1
        self.old_key = -1
        # Initialize mouse
        self.mouse = ()
        # Initialize signal
        self.signal = True
        # Catch all signals
        for sig in [signal.SIGTERM, signal.SIGINT, signal.SIGHUP, signal.SIGQUIT]:
            signal.signal(sig, self.handler)
        # Run the GUI
        if start:
            self.run(loop, seconds)

    def handler(self, signum=None, frame=None):
        logger.info("Signal handler called with signal {signum}".format(signum=signum))
        # Close gui
        self.signal = False

    def run(self, loop, seconds):
        # In this program, we don't want keystrokes echoed to the console,
        # so we run this to disable that
        curses.noecho()
        # Additionally, we want to make it so that the user does not have to press
        # enter to send keys to our program, so here is how we get keys instantly
        curses.cbreak()
        # Try to hide the cursor
        if hasattr(curses, 'curs_set'):
            try:
                curses.curs_set(0)
            except Exception:
                pass
        # Lastly, keys such as the arrow keys are sent as funny escape sequences to
        # our program. We can make curses give us nicer values (such as curses.KEY_LEFT)
        # so it is easier on us.
        self.stdscr.keypad(True)
        # Enable mouse mask
        _, _ = curses.mousemask(curses.BUTTON1_CLICKED)
        # Refreshing page curses loop
        # https://stackoverflow.com/questions/54409978/python-curses-refreshing-text-with-a-loop
        self.stdscr.nodelay(1)
        # Using current time
        old = datetime.now()
        # Here is the loop of our program, we keep clearing and redrawing in this loop
        while not self.events() and self.signal:
            # Draw pages
            self.draw()
            # Increase page automatically if loop enabled
            if loop and datetime.now() - old >= timedelta(seconds=seconds):
                self.increase(loop=True)
                old = datetime.now()

    @check_size(20, 50)
    def draw(self):
        # First, clear the screen
        self.stdscr.erase()
        # Write head of the jtop
        self.header()
        # Get page selected
        page = self.pages[self.n_page]
        # Draw the page
        page.draw(self.key, self.mouse)
        # Draw menu
        self.menu()
        # Draw the screen
        self.stdscr.refresh()
        # Set a timeout and read keystroke
        self.stdscr.timeout(self.refresh)

    def increase(self, loop=False):
        # check reset
        if loop and self.n_page >= len(self.pages) - 1:
            idx = 0
        else:
            idx = self.n_page + 1
        # Fix set new page
        self.set(idx + 1)

    def decrease(self, loop=False):
        # check reset
        if loop and self.n_page <= 0:
            idx = len(self.pages) + 1
        else:
            idx = self.n_page + 1
        self.set(idx - 1)

    def set(self, idx):
        if idx <= len(self.pages) and idx > 0:
            self.n_page = idx - 1

    @check_curses
    def header(self):
        # Title script
        # Reference: https://stackoverflow.com/questions/25872409/set-gnome-terminal-window-title-in-python
        # Print jtop basic info
        set_xterm_title("jtop" + xterm_line(self.jetson))
        # Write first line
        board = self.jetson.board["info"]
        # Add extra Line if without sudo
        idx = 0
        if self.jetson.userid != 0:
            _, width = self.stdscr.getmaxyx()
            self.stdscr.addstr(0, 0, ("{0:<" + str(width) + "}").format(" "), curses.color_pair(11))
            string_sudo = "SUDO SUGGESTED"
            self.stdscr.addstr(0, (width - len(string_sudo)) // 2, string_sudo, curses.color_pair(11))
            idx = 1
        self.stdscr.addstr(idx, 0, board["Machine"] + " - Jetpack " + board["Jetpack"], curses.A_BOLD)

    @check_curses
    def menu(self):
        height, width = self.stdscr.getmaxyx()
        # Set background for all menu line
        self.stdscr.addstr(height - 1, 0, ("{0:<" + str(width - 1) + "}").format(" "), curses.A_REVERSE)
        position = 1
        for idx, page in enumerate(self.pages):
            color = curses.A_NORMAL if self.n_page == idx else curses.A_REVERSE
            self.stdscr.addstr(height - 1, position, str(idx + 1), color | curses.A_BOLD)
            self.stdscr.addstr(height - 1, position + 1, page.name + " ", color)
            position += len(page.name) + 3
        self.stdscr.addstr(height - 1, position, "Q", curses.A_REVERSE | curses.A_BOLD)
        self.stdscr.addstr(height - 1, position + 1, "uit ", curses.A_REVERSE)
        # Author name
        name_author = "Raffaello Bonghi"
        self.stdscr.addstr(height - 1, width - len(name_author), name_author, curses.A_REVERSE)

    def event_menu(self, mx, my):
        height, _ = self.stdscr.getmaxyx()
        # Check if is an event menu
        if my == height - 1:
            # Check which page
            position = 1
            for idx, page in enumerate(self.pages):
                size = len(page.name) + 3
                # Check if mouse is inside menu name
                if mx >= position and mx < position + size:
                    # Set new page
                    self.set(idx + 1)
                    return False
                # Increase counter
                position += size
            # Quit button
            if mx >= position and mx < position + 4:
                return True
        return False

    def events(self):
        event = self.stdscr.getch()
        # Run keyboard check
        status_mouse = False
        status_keyboard = self.keyboard(event)
        # Clear event mouse
        self.mouse = ()
        # Check event mouse
        if event == curses.KEY_MOUSE:
            try:
                _, mx, my, _, _ = curses.getmouse()
                # Run event menu controller
                status_mouse = self.event_menu(mx, my)
                self.mouse = (mx, my)
            except curses.error:
                pass
        return status_keyboard or status_mouse

    def keyboard(self, event):
        self.key = event
        if self.old_key != self.key:
            # keyboard check list
            if self.key == curses.KEY_LEFT:
                self.decrease(loop=True)
            elif self.key == curses.KEY_RIGHT or self.key == ord('\t'):
                self.increase(loop=True)
            elif self.key in [ord(str(n)) for n in range(10)]:
                num = int(chr(self.key))
                self.set(num)
            elif self.key == ord('q') or self.key == ord('Q') or self.ESC_BUTTON(self.key):
                # keyboard check quit button
                return True
            # Store old value key
            self.old_key = self.key
        return False

    def ESC_BUTTON(self, key):
        """
            Check there is another character prevent combination ALT + <OTHER CHR>
            https://stackoverflow.com/questions/5977395/ncurses-and-esc-alt-keys
        """
        if key == 27:
            n = self.stdscr.getch()
            if n == -1:
                return True
        return False
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import re
import os
# Logging
import logging
# Launch command
import subprocess as sp

# Create logger for jplotlib
logger = logging.getLogger(__name__)


class JetsonClocks(object):
    """
        This controller manage the jetson_clocks service.
    """
    # CPU Cluster regex
    # CPU Cluster Switching: Disabled
    CPU_CLUSTER_REGEXP = re.compile(r'CPU Cluster Switching: ((.*))')
    # CPU regex
    # NANO: cpu0: Online=1 Governor=schedutil MinFreq=102000 MaxFreq=1428000 CurrentFreq=1428000 IdleStates: WFI=1 c7=1
    # Xavier: cpu0: Online=1 Governor=schedutil MinFreq=1190400 MaxFreq=2265600 CurrentFreq=1574400 IdleStates: C1=1 c6=1
    CPU_REGEXP = re.compile(r'cpu(.+?): Online=(.+?) Governor=(.+?) MinFreq=(.+?) MaxFreq=(.+?) CurrentFreq=(.+?) IdleStates: ((.*))')
    # GPU regex
    # GPU MinFreq=76800000 MaxFreq=921600000 CurrentFreq=384000000
    GPU_REGEXP = re.compile(r'GPU MinFreq=(.+?) MaxFreq=(.+?) CurrentFreq=((.*))')
    # EMC regex
    # EMC MinFreq=204000000 MaxFreq=1600000000 CurrentFreq=1600000000 FreqOverride=0
    EMC_REGEXP = re.compile(r'EMC MinFreq=(.+?) MaxFreq=(.+?) CurrentFreq=(.+?) FreqOverride=((.*))')

    class JCException(Exception):
        pass

    def __init__(self, config_file, service_name='jetson_performance'):
        # Config file
        self.config_file = config_file + "/l4t_dfs.conf"
        # Service
        self.service_name = service_name
        self.last_status = ""
        # Jetson Clocks path
        if os.path.isfile("/usr/bin/jetson_clocks"):
            self.jc_bin = "/usr/bin/jetson_clocks"
        elif os.path.isfile("/home/nvidia/jetson_clocks.sh"):
            self.jc_bin = "/home/nvidia/jetson_clocks.sh"
        else:
            raise JetsonClocks.JCException("No jetson_clock script is availble in this board")
        # Store configuration if does not exist
        self.store()

    def show(self):
        p = sp.Popen([self.jc_bin, '--show'], stdout=sp.PIPE, stderr=sp.PIPE)
        out, _ = p.communicate()
        # Decode lines
        lines = out.decode("utf-8")
        # Load lines
        status = {"cpu": {}}
        for line in lines.split("\n"):
            # Search configuration CPU config
            match = JetsonClocks.CPU_REGEXP.search(line)
            # if match extract name and number
            if match:
                # Load CPU information
                cpu = {"Online": True if int(match.group(2)) == 1 else False,
                       "Governor": str(match.group(3)),
                       "MinFreq": int(match.group(4)),
                       "MaxFreq": int(match.group(5)),
                       "CurrentFreq": int(match.group(6)),
                       "IdleStates": {str(state.split("=")[0]): int(state.split("=")[1]) for state in match.group(7).split()}}
                # Store in CPU list
                status["cpu"]["cpu{num}".format(num=match.group(1))] = cpu
                continue
            # Search configuration GPU config
            match = JetsonClocks.GPU_REGEXP.search(line)
            # Load GPU match
            if match:
                status["gpu"] = {"MinFreq": int(match.group(1)),
                                 "MaxFreq": int(match.group(2)),
                                 "CurrentFreq": int(match.group(3))}
                continue
            # Search configuration EMC config
            match = JetsonClocks.EMC_REGEXP.search(line)
            # Load EMC match
            if match:
                status["emc"] = {"MinFreq": int(match.group(1)),
                                 "MaxFreq": int(match.group(2)),
                                 "CurrentFreq": int(match.group(3)),
                                 "FreqOverride": int(match.group(4))}
                continue
            # Search configuration CPU Cluster config
            match = JetsonClocks.CPU_CLUSTER_REGEXP.search(line)
            # Load EMC match
            if match:
                status["cluster"] = str(match.group(1))
                continue
            # All other lines of jetson_clocks show are skipped
            # SOC family:tegra210  Machine:NVIDIA Jetson Nano Developer Kit
            # Online CPUs: 0-3
            # Fan: speed=0
            # NV Power Mode: MAXN
        return status

    @property
    def service(self):
        p = sp.Popen(['systemctl', 'is-active', self.service_name + '.service'], stdout=sp.PIPE, stderr=sp.PIPE)
        out, _ = p.communicate()
        return str(out.decode("utf-8")).strip()

    @property
    def status(self):
        # Load jetson_clocks show
        show = self.show()
        # Make statistics
        stat = []
        if 'cpu' in show:
            for cpu in show['cpu'].values():
                # Check status CPUs
                stat += [cpu['MaxFreq'] == cpu['MinFreq']]
                stat += [cpu['MaxFreq'] == cpu['CurrentFreq']]
        # Check status GPU
        if 'gpu' in show:
            gpu = show['gpu']
            stat += [gpu['MaxFreq'] == gpu['MinFreq']]
            stat += [gpu['MaxFreq'] == gpu['CurrentFreq']]
        # Don't need to check EMC frequency
        # Check status EMC
        # if 'emc' in show:
        #     emc = show['emc']
        #     stat += [emc['MaxFreq'] == emc['MinFreq']]
        #     stat += [emc['MaxFreq'] == emc['CurrentFreq']]
        if not stat:
            raise JetsonClocks.JCException("Require super user")
        return all(stat)

    @property
    def start(self):
        start_val = True if self.service == "active" else False
        return start_val

    @start.setter
    def start(self, value):
        if not isinstance(value, bool):
            raise Exception("Use a boolean")
        # Do not run if jetson_clocks as already running
        if self.status and value:
            return
        # make service script
        start_val = "start" if value else "stop"
        p = sp.Popen(['systemctl', start_val, self.service_name + '.service'], stdout=sp.PIPE, stderr=sp.PIPE)
        _, err = p.communicate()
        self.last_status = err.decode("utf-8")

    @property
    def enable(self):
        p = sp.Popen(['systemctl', 'is-enabled', self.service_name + '.service'], stdout=sp.PIPE, stderr=sp.PIPE)
        out, _ = p.communicate()
        enable_val = True if str(out.decode("utf-8")).strip() == "enabled" else False
        return enable_val

    @enable.setter
    def enable(self, value):
        if not isinstance(value, bool):
            raise Exception("Use a boolean")
        enable_val = "enable" if value else "disable"
        p = sp.Popen(['systemctl', enable_val, self.service_name + '.service'], stdout=sp.PIPE, stderr=sp.PIPE)
        _, err = p.communicate()
        self.last_status = err.decode("utf-8")

    def store(self):
        if not os.path.isfile(self.config_file):
            p = sp.Popen([self.jc_bin, '--store', self.config_file], stdout=sp.PIPE)
            out, _ = p.communicate()
            # Extract result
            if out.decode("utf-8"):
                return True
            else:
                return False
        else:
            return False

    def clear(self):
        if os.path.isfile(self.config_file):
            # Remove configuration file
            os.remove(self.config_file)
            return True
        else:
            return False

# EOF



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2020 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# Jtop server
from .service import JtopServer


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    @staticmethod
    def ok(message="OK"):
        return bcolors.OKGREEN + message + bcolors.ENDC

    @staticmethod
    def warning(message="WARN"):
        return bcolors.WARNING + message + bcolors.ENDC

    @staticmethod
    def fail(message="ERR"):
        return bcolors.FAIL + message + bcolors.ENDC

# https://stackoverflow.com/questions/11114589/creating-hidden-arguments-with-python-argparse
def main():
    # jtop service
    server = JtopServer()
    print("Service started")
    try:
        server.loop()
    except KeyboardInterrupt:
        pass
    # Close stats server
    print("Close service")
    server.close()


if __name__ == "__main__":
    main()
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


from copy import deepcopy
from datetime import timedelta
# control command line
import curses
# Graphics elements
from .lib.common import (check_curses,
                         strfdelta,
                         label_freq,
                         plot_name_info,
                         size_min)
from .lib.linear_gauge import linear_gauge, GaugeName
from ..core.jetson_clocks import JetsonClocks


@check_curses
def plot_CPUs(stdscr, offest, list_cpus, width):
    max_bar = int(float(width) / 2.0)
    for idx, cpu in enumerate(list_cpus):
        # Split in double list
        start = max_bar if idx >= len(list_cpus) / 2 and len(list_cpus) > 4 else 0
        off_idx = idx - len(list_cpus) / 2 if idx >= len(list_cpus) / 2 and len(list_cpus) > 4 else idx
        # Check if exist governor and add in percent name
        percent = ""
        if 'val' in cpu and 'governor' in cpu:
            percent = "{gov} -{val: 4}%".format(gov=cpu['governor'].capitalize(), val=cpu['val'])
        # Show linear gauge
        linear_gauge(stdscr, offset=int(offest + off_idx), start=start, size=max_bar,
                     name=GaugeName(cpu['name'], color=curses.color_pair(6)),
                     value=cpu.get('val', 0),
                     status=cpu['status'],
                     percent=percent,
                     label=label_freq(cpu))
    if len(list_cpus) > 4:
        return int(offest + idx / 2 + 1)
    else:
        return int(offest + idx + 1)


@check_curses
def plot_temperatures(stdscr, start, offset, width, height, jetson):
    start = start + (width - 17) // 2
    # Define color temperatures
    color_options = {
        60: curses.color_pair(1),
        40: curses.color_pair(3),
        20: curses.A_NORMAL,
    }
    list_options = sorted(color_options.keys(), reverse=True)
    # Plot title
    stdscr.addstr(offset, start - 1, " [Sensor] ", curses.A_BOLD)
    stdscr.addstr(offset, start + 11, " [Temp] ", curses.A_BOLD)
    # Plot name and temperatures
    temps = deepcopy(jetson.stats['TEMP'])
    if 'PMIC' in temps:
        del temps['PMIC']
    for idx, temp in enumerate(sorted(temps)):
        # Print temperature name
        value = jetson.stats['TEMP'][temp]
        stdscr.addstr(offset + idx + 1, start, ("{name:<7}").format(name=temp))
        # Set color temperature
        color = curses.A_NORMAL
        for k in list_options:
            if value >= k:
                color = color_options[k]
                break
        # Print temperature value
        stdscr.addstr(offset + idx + 1, start + offset // 2 + 3, ("{val:8.2f}C").format(val=value), color)


@check_curses
def plot_watts(stdscr, start, offset, width, height, jetson):
    # List of all watts
    watts = deepcopy(jetson.stats['WATT'])
    # In according with:
    # https://forums.developer.nvidia.com/t/power-consumption-monitoring/73608/8
    # https://github.com/rbonghi/jetson_stats/issues/51
    total_name = ""
    for val in watts:
        if "_IN" in val:
            total_name = val
            break
    # Extract the total from list
    # Otherwise sum all values
    # Example for Jetson Xavier
    # https://forums.developer.nvidia.com/t/xavier-jetson-total-power-consumption/81016
    if total_name:
        total = watts[total_name]
        del watts[total_name]
    else:
        total = {'cur': 0, 'avg': 0}
    # Plot title
    center_column = width // 2
    stdscr.addstr(offset, start, " [Power/mW] ", curses.A_BOLD)
    stdscr.addstr(offset, start + center_column - 1, " [Cur] ", curses.A_BOLD)
    stdscr.addstr(offset, start + width - 8, " [Avr] ", curses.A_BOLD)
    # Plot watts
    for idx, watt in enumerate(sorted(watts)):
        value = jetson.stats['WATT'][watt]
        watt_name = watt.replace("VDD_", "").replace("POM_", "").replace("_", " ")
        if not total_name:
            total['cur'] += int(value['cur'])
            total['avg'] += int(value['avg'])
        stdscr.addstr(offset + idx + 1, start + 1, watt_name, curses.A_NORMAL)
        stdscr.addstr(offset + idx + 1, start + center_column, str(value['cur']), curses.A_NORMAL)
        stdscr.addstr(offset + idx + 1, start + width - 7, str(value['avg']), curses.A_NORMAL)
    # Fix total_name if empty
    if not total_name:
        total_name = "Total"
    else:
        total_name = total_name.replace("VDD_", "").replace("POM_", "").replace("_", " ")
    # Plot totals before finishing
    stdscr.addstr(offset + idx + 2, start + 1, total_name, curses.A_BOLD)
    stdscr.addstr(offset + idx + 2, start + center_column, str(total['cur']), curses.A_BOLD)
    stdscr.addstr(offset + idx + 2, start + width - 7, str(total['avg']), curses.A_BOLD)


@check_curses
def compact_info(stdscr, start, offset, width, height, jetson):
    # Title menu
    stdscr.addstr(offset, start + (width - 7) // 2, " [info] ", curses.A_BOLD)
    counter = 1
    # Model board information
    uptime_string = strfdelta(timedelta(seconds=jetson.uptime), "{days} days {hours}:{minutes}:{seconds}")
    plot_name_info(stdscr, offset + counter, start + 1, "UpT", uptime_string)
    counter += 1
    # FAN status
    if "FAN" in jetson.stats:
        fan = jetson.stats["FAN"]
        if 'cpwm' in fan:
            if 'ctrl' in fan:
                ctrl = "Ta" if fan.get("ctrl", False) else "Tm"
            else:
                ctrl = "T"
            label = "{ctrl}={target: >3}%".format(ctrl=ctrl, target=fan.get("tpwm", 0))
            value = fan.get('cpwm', 0)
        else:
            label = ''
            value = fan.get('tpwm', 0)
        linear_gauge(stdscr, offset=offset + counter, start=start + 1, size=width,
                     name=GaugeName('FAN', color=curses.color_pair(6)),
                     value=value,
                     label=label,
                     status=fan['status'])
    else:
        stdscr.hline(offset + counter, start + 1, curses.ACS_BLOCK, width, curses.color_pair(3))
        stdscr.addstr(offset + counter, start + (width - 8) // 2, " NO FAN ", curses.color_pair(3))
    counter += 1
    # Jetson clocks status
    jc = jetson.jetson_clocks
    if jc is not None:
        jc_service = jc.service
        jc_enable = jc.enable
        try:
            jc_status = jc.status
            # Running (Green) or Normal (Grey)
            color = curses.color_pair(2) if jc_status else curses.A_NORMAL
            # Write status jetson_clocks
            jc_status_name = "Running" if jc_status else "Stopped"
            # Specify the service running
            if jc_service == "active":
                jc_status_name += "+"
            # Show if JetsonClock is enabled or not
            if jc_enable:
                jc_status_name = "[" + jc_status_name + "]"
        except JetsonClocks.JCException:
            # Fix error color
            color = curses.color_pair(11)
            # SUDO SUGGESTED is too long, change with a series of spaces
            # The number 16 = len("jetson clocks: ") + 1
            jc_status_name = (width - 16) * " "
        # Show status jetson_clocks
        plot_name_info(stdscr, offset + counter, start + 1, "Jetson clocks", jc_status_name, color)
        counter += 1
    # NVP Model
    nvpmodel = jetson.nvpmodel
    if nvpmodel is not None:
        plot_name_info(stdscr, offset + counter, start + 1, "NV Power[" + str(nvpmodel.num) + "]", nvpmodel.mode)
        counter += 1
    # Write all engines
    engines(stdscr, start, offset + counter, width, height, jetson)


def engines(stdscr, start, offset, width, height, jetson):
    stdscr.hline(offset, start + 1, curses.ACS_HLINE, width - 1)
    stdscr.addstr(offset, start + (width - 13) // 2, " [HW engines] ", curses.A_BOLD)
    counter = 1
    # APE frequency
    if 'APE' in jetson.stats:
        plot_name_info(stdscr, offset + counter, start + 1, "APE", str(jetson.stats['APE']['val']) + "MHz")
        counter += 1
    # Find encoders
    if 'NVENC' in jetson.stats:
        enc_name = 'NVENC'
        enc_val = "{value}{unit}Hz".format(value=jetson.stats['NVENC']['val'], unit="M")
    elif 'MSENC' in jetson.stats:
        enc_name = 'MSENC'
        enc_val = "{value}{unit}Hz".format(value=jetson.stats['MSENC']['val'], unit="M")
    else:
        enc_name = 'NVENC'
        enc_val = "[OFF]"
    # Find decoders
    if 'NVDEC' in jetson.stats:
        dec_name = 'NVDEC'
        dec_val = "{value}{unit}Hz".format(value=jetson.stats['NVDEC']['val'], unit="M")
    else:
        dec_name = 'NVDEC'
        dec_val = "[OFF]"
    double_info(stdscr, start + 1, offset + counter, width, (enc_name, enc_val), (dec_name, dec_val))
    counter += 1
    # NVJPG
    nvjpg = jetson.nvjpg
    if nvjpg:
        status = nvjpg['status']
        if status:
            value, _, unit = size_min(nvjpg['rate'])
            value = "{value}{unit}Hz".format(value=value, unit=unit)
        else:
            value = "[OFF]"
        # Plot status
        plot_name_info(stdscr, offset + counter, start + 1, "NVJPG", value)


def double_info(stdscr, start, offset, width, enc, dec):
    plot_name_info(stdscr, offset, start, enc[0], enc[1])
    plot_name_info(stdscr, offset, start + width // 2, dec[0], dec[1])
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# Math functions
from math import ceil
import curses
from curses.textpad import rectangle
from collections import deque
from .common import check_curses


class Chart(object):
    """
    Chart draw object
    http://www.melvilletheatre.com/articles/ncurses-extended-characters/index.html
    """
    def __init__(self, jetson, name, interval, callback, type_value=int, line="*", color=curses.A_NORMAL, color_chart=[], fill=True, time=10.0, tik=2):
        self.jetson = jetson
        self.name = name
        self.callback = callback
        # Set shape
        self.refresh = interval
        # Design chart shape
        self.line = line
        self.color = color
        self.color_chart = color_chart if color_chart else [color]
        self.fill = fill
        # Set timing
        self.time = time
        self.tik = tik
        # Initialization chart
        max_record = int(self.time * (float(1.0 / float(interval)) * 1000.0))
        self.values = deque(max_record * [(len(self.color_chart) * [0])], maxlen=max_record)
        # Initialzie default values and unit
        self.unit = "%"
        self.type_value = type_value
        self.max_val = 100
        self.active = True
        self.message = "OFF"
        # Attach the chart for every update from jtop
        jetson.attach(self)

    def statusChart(self, active, message):
        self.active = active
        self.message = message

    def update(self, jetson):
        """ Local update chart """
        # Call the callback
        data = self.callback(jetson, self.name)
        # Get max value if is present
        self.max_val = data.get("max", self.max_val)
        # Get unit
        self.unit = data.get("unit", self.unit)
        # Get status
        self.active = data.get("active", self.active)
        # update the queue
        value = data.get("value", [0])
        self.values.append(value)

    @check_curses
    def draw(self, stdscr, size_x, size_y, label="", y_label=True):
        # Evaluate Diplay X, and Y size
        displayX = size_x[1] - size_x[0] + 1
        displayY = size_y[1] - size_y[0] - 1
        # Text label
        stdscr.addstr(size_y[0], size_x[0], self.name, curses.A_BOLD)
        if label:
            stdscr.addstr(size_y[0], size_x[0] + len(self.name) + 1, label, self.color | curses.A_BOLD)
        # Plot chart lines
        if self.active:
            # Plot values
            self._plot_values(stdscr, size_x, size_y, displayX, displayY, label=y_label)
        else:
            l_label = size_x[1] - 6 if y_label else size_x[1] - 1
            rectangle(stdscr, size_y[0] + 1, size_x[0], size_y[1] - 2, l_label)
            # Write message
            middle_x = (l_label - size_x[0] - len(self.message)) // 2
            middle_y = (size_y[1] - size_y[0]) // 2
            stdscr.addstr(size_y[0] + middle_y, size_x[0] + middle_x, self.message, curses.A_BOLD)
        # Draw ticks and labels
        self._plot_x_axis(stdscr, size_x, size_y, displayX, label=y_label)
        # Plot chart shape and labels
        self._plot_y_axis(stdscr, size_x, size_y, displayY, label=y_label)

    def _plot_y_axis(self, stdscr, size_x, size_y, displayY, label=True):
        # Plot chart shape and labels
        label_x = size_x[1] - 5 if label else size_x[1]
        for point in range(displayY):
            if displayY != point:
                value_n = self.max_val / float(displayY - 1) * float(displayY - point - 1)
                try:
                    stdscr.addch(1 + size_y[0] + point, label_x, curses.ACS_LTEE)
                    if not label:
                        continue
                    if self.type_value == float:
                        lab_c = "{value:2.1f}{unit}".format(value=value_n, unit=self.unit)
                    else:
                        lab_c = "{value:3d}{unit}".format(value=int(value_n), unit=self.unit)
                    stdscr.addstr(1 + size_y[0] + point, label_x + 2, lab_c, curses.A_BOLD)
                except curses.error:
                    pass

    def _plot_x_axis(self, stdscr, size_x, size_y, displayX, label=True):
        val = float(displayX - 2) / float(len(self.values))
        # Draw ticks and labels
        ten_sec = int(self.tik * 1000 / self.refresh)
        counter = 0
        label_y = size_x[1] - 5 if label else size_x[1]
        # Draw line
        stdscr.hline(size_y[1] - 1, size_x[0], curses.ACS_HLINE, label_y - size_x[0])
        for point in range(displayX):
            x_val = label_y - point
            if x_val >= size_x[0]:
                try:
                    # Draw tick label
                    if ((point) / ceil(val)) % ten_sec == 0:
                        stdscr.addch(size_y[1] - 1, x_val, curses.ACS_TTEE)
                    # Draw label
                    if counter > 0 and ((point - 1) / ceil(val)) % ten_sec == 0:
                        stdscr.addstr(size_y[1], x_val, "-{time}s".format(time=self.tik * counter))
                    elif counter == 0 and ((point - 1) / ceil(val)) % ten_sec == 0:
                        if label:
                            stdscr.addstr(size_y[1], x_val + 3, "time")
                        stdscr.addstr(size_y[1], x_val + 1, "0")
                    # Increase counter
                    if ((point - 1) / ceil(val)) % ten_sec == 0:
                        counter += 1
                except curses.error:
                    pass

    def _plot_values(self, stdscr, size_x, size_y, displayX, displayY, label=True):
        """ Plot values """
        val = float(displayX - 2) / float(len(self.values))
        time_block = int(ceil(val))

        label_x = size_x[1] - 5 if label else size_x[1]

        for idx, values in enumerate(reversed(self.values)):
            # TODO: n = n if n <= self.max_val else self.max_val
            x_val = label_x - idx * time_block - 3
            # Draw chart
            counter = 0
            for value, color in zip(values, self.color_chart):
                y_val = int((float(displayY - 1) / self.max_val) * value)

                if x_val >= size_x[0]:
                    if self.fill:
                        for n in range(counter + 1, counter + y_val + 1):
                            try:
                                stdscr.addstr(size_y[1] - 1 - n, x_val, " " * time_block, color)
                            except curses.error:
                                pass
                    else:
                        try:
                            stdscr.addstr(size_y[1] - 1 - y_val, x_val, self.line, self.color)
                        except curses.error:
                            pass
                counter += y_val
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# control command line
import curses
from curses.textpad import rectangle
from threading import Thread
# Size button
SIZE_BUTTON_HEIGHT = 2


class Button:

    def __init__(self, stdscr, key, label="", action=None, underline=True, keyboard=True):
        self.stdscr = stdscr
        self.key = key
        self.label = label
        self.action = action
        self.underline = underline
        self.keyboard = keyboard

    def draw(self, posy, posx, key, mouse, color=curses.A_REVERSE, exstatus=False):
        # Draw rectangle
        width = self.sizeX()
        rectangle(self.stdscr, posy, posx, posy + SIZE_BUTTON_HEIGHT, posx + width)
        # status
        status = self._keyPressed(key) or self._mousePressed(posy, posx, width, mouse)
        # Write key letter
        if self.key:
            underline = curses.A_UNDERLINE if self.underline else curses.A_NORMAL
            pressed = color if status or exstatus and not self.label else curses.A_NORMAL
            self.stdscr.addstr(posy + 1, posx + 2, self.key, underline | pressed)
        # Write label
        if self.label:
            posx_label = 4 if self.key else 2
            pressed = color if status or exstatus else curses.A_NORMAL
            self.stdscr.addstr(posy + 1, posx + posx_label, self.label, pressed)
        # Run action
        if status and self.action is not None:
            # Run a thread
            th = Thread(target=self.action, args=(self.key))
            th.start()

    def sizeX(self):
        width = 4 if self.key else 2
        return width + len(self.label) + 1 if self.label else width

    def _keyPressed(self, key):
        # Default status
        if not self.keyboard:
            return False
        return key == ord(self.key)

    def _mousePressed(self, posy, posx, width, mouse):
        if mouse:
            mx, my = mouse
            if my >= posy and my <= posy + SIZE_BUTTON_HEIGHT and \
               mx >= posx and mx <= posx + width:
                return True
        return False


class ButtonList:

    def __init__(self, stdscr, keys, labels, action=None):
        self.stdscr = stdscr
        # List of buttons
        self.buttons = []
        for key, label in zip(keys, labels):
            key = str(key) if not isinstance(key, str) else key
            self.buttons += [Button(stdscr, key, label=label, underline=False, keyboard=False, action=action)]

    def draw(self, posy, posx, width, key, mouse, lstatus=[], select=-1):
        # Box max size
        # rectangle(self.stdscr, posy, posx, posy + SIZE_BUTTON_HEIGHT, posx + width)
        # draw buttons
        x_line = 0
        y_line = 0
        lstatus = lstatus if lstatus else [False] * len(self.buttons)
        for idx, (button, status) in enumerate(zip(self.buttons, lstatus)):
            # Find next position
            b_width = button.sizeX()
            if x_line + b_width >= width:
                y_line += SIZE_BUTTON_HEIGHT + 1
                x_line = 0
            # Draw button
            color = curses.A_REVERSE if status else curses.color_pair(1)
            selected = True if select == idx else not status
            button.draw(posy + y_line, posx + x_line, key, mouse, color=color, exstatus=selected)
            # Increase x counter
            x_line += b_width + 1
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import sys
# control command line
import curses
# Functions and decorators
from functools import wraps


def xterm_line(jetson):
    str_xterm = " " + jetson.board["board"]["TYPE"]
    nvp = jetson.nvpmodel
    # if "GR3D" in jetson.stats:
    #     gpu = jetson.stats["GR3D"]['val']
    #     str_xterm += " - GPU {gpu: 3}% {label}".format(gpu=gpu, label=label_freq(jetson.stats["GR3D"]))
    if nvp is not None:
        str_xterm += " - {}".format(nvp.mode)
    return str_xterm


def set_xterm_title(title):
    '''
    Set XTerm title using escape sequences.
    By default, sets as 'Python' and the version number.
    '''
    # Make sure this terminal supports the OSC code (\33]),
    # though not necessarily that it supports setting the title.
    # If this check causes compatibility issues, you can add
    # items to the tuple, or remove the check entirely.
    if os.environ.get('TERM') in ('xterm',
                                  'xterm-color',
                                  'xterm-256color',
                                  'linux',
                                  'screen',
                                  'screen-256color',
                                  'screen-bce',
                                  ):
        sys.stdout.write('\33]0;' + title + '\a')
        sys.stdout.flush()


def check_size(height_max, width_max):
    """ Check curses size window """
    def check_size_window(func):
        @wraps(func)
        def wrapped(self, *args, **kwargs):
            # Extract window size
            height, width = self.stdscr.getmaxyx()
            # Check size window
            if width >= width_max and height >= height_max:
                return func(self, *args, **kwargs)
            else:
                string_warning = "jtop"
                string_warning_msg = "Change size window!"
                size_window_width = "Width: " + str(width) + " >= " + str(width_max)
                size_window_height = "Height: " + str(height) + " >= " + str(height_max)
                try:
                    height_c = int(height / 2)
                    self.stdscr.addstr(height_c - 2, int((width - len(string_warning)) / 2), string_warning, curses.A_BOLD)
                    self.stdscr.addstr(height_c - 1, int((width - len(string_warning_msg)) / 2), string_warning_msg, curses.A_BOLD)
                    # Show size window
                    if width < width_max:
                        self.stdscr.addstr(height_c, int((width - len(size_window_width)) / 2), str(size_window_width), curses.color_pair(1))
                    else:
                        size_window_width = "Width OK!"
                        self.stdscr.addstr(height_c, int((width - len(size_window_width)) / 2), size_window_width, curses.A_BOLD)
                    if height < height_max:
                        self.stdscr.addstr(height_c + 1, int((width - len(size_window_height)) / 2), str(size_window_height), curses.color_pair(1))
                    else:
                        size_window_height = "Height OK!"
                        self.stdscr.addstr(height_c + 1, int((width - len(size_window_height)) / 2), str(size_window_height), curses.A_BOLD)
                    # Set background for all menu line
                    self.stdscr.addstr(height - 1, 0, ("{0:<" + str(width - 1) + "}").format(" "), curses.A_REVERSE)
                    # Add close option menu
                    self.stdscr.addstr(height - 1, 1, "Q to close", curses.A_REVERSE)
                except curses.error:
                    pass
        return wrapped
    return check_size_window


def check_curses(func):
    """ Check curses write """
    @wraps(func)
    def wrapped(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except curses.error:
            pass
    return wrapped


def strfdelta(tdelta, fmt):
    """ Print delta time
        - https://stackoverflow.com/questions/8906926/formatting-python-timedelta-objects
    """
    d = {"days": tdelta.days}
    d["hours"], rem = divmod(tdelta.seconds, 3600)
    d["minutes"], d["seconds"] = divmod(rem, 60)
    return fmt.format(**d)


def label_freq(value):
    if 'frq' in value:
        freq = value['frq']
        if freq >= 1000:
            return "{0:2.1f}GHz".format(freq / 1000.0)
        else:
            return str(freq) + "MHz"
    else:
        return ""


def size_min(num, divider=1.0, n=0, start=''):
    if num >= divider * 1000.0:
        n += 1
        divider *= 1000.0
        return size_min(num, divider, n, start)
    else:
        vect = ['', 'k', 'M', 'G', 'T']
        idx = vect.index(start)
        return round(num / divider, 1), divider, vect[n + idx]


@check_curses
def plot_dictionary(stdscr, offset, data, name, start=0):
    # Plot title
    stdscr.addstr(offset, start, name + ":", curses.A_BOLD)
    counter = 1
    for key, value in data.items():
        if 'text' in value:
            stdscr.addstr(offset + counter, start, " {0:<10} {1}".format(key, value['text']))
        else:
            stdscr.addstr(offset + counter, start, " {0:<10} {1}".format(key, value))
        counter += 1


@check_curses
def plot_name_info(stdscr, offset, start, name, value, color=curses.A_NORMAL):
    stdscr.addstr(offset, start, name + ":", curses.A_BOLD)
    stdscr.addstr(offset, start + len(name) + 2, value, color)
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os
import stat
import socket
from  grp import getgrnam
from .core import Tegrastats


class JtopServer:

    PIPE_JTOP_CTRL = '/tmp/jtop_ctrl'
    PIPE_JTOP_STATS = '/tmp/jtop_stats'
    PIPE_JTOP_USER = 'jetson_stats'

    def __init__(self, timeout=1):
        try:
            gid = getgrnam(JtopServer.PIPE_JTOP_USER).gr_gid
        except KeyError:
            # TODO: Check how to be writeable only from same group
            # raise Exception("Group jetson_stats does not exist!")
            print("Check how to be writeable only from same group")
            gid = os.getgid()
        # Remove old pipes if exists
        if os.path.exists(JtopServer.PIPE_JTOP_CTRL):
            print("Remove old pipe {pipe}".format(pipe=JtopServer.PIPE_JTOP_CTRL))
            os.remove(JtopServer.PIPE_JTOP_CTRL)
        if os.path.exists(JtopServer.PIPE_JTOP_STATS):
            print("Remove old pipe {pipe}".format(pipe=JtopServer.PIPE_JTOP_STATS))
            os.remove(JtopServer.PIPE_JTOP_STATS)
        # Initialize and bind control socket
        self.sock_ctrl = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        self.sock_ctrl.bind(JtopServer.PIPE_JTOP_CTRL)
        self.sock_ctrl.settimeout(timeout)
        os.chown(JtopServer.PIPE_JTOP_CTRL, os.getuid(), gid)
        # Set mode
        # https://www.tutorialspoint.com/python/os_chmod.htm
        os.chmod(JtopServer.PIPE_JTOP_CTRL, stat.S_IWOTH)
        # Initialize and bind statistics socket
        self.socket_stats = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
        self.socket_stats.bind(JtopServer.PIPE_JTOP_STATS)
        # Set ownership file
        os.chown(JtopServer.PIPE_JTOP_STATS, os.getuid(), gid)
        # Set mode
        # TODO: Set mode is only readable from all
        # os.chmod(JtopServer.PIPE_JTOP_CTRL, stat.S_IWOTH)
        # Setup tegrastats
        self.tegra = Tegrastats('/usr/bin/tegrastats')
        self.tegra.attach(self.tegra_stats)

    def loop(self):
        while True:
            try:
                datagram = self.sock_ctrl.recv(1024)
                print("Datagram: {datagram}".format(datagram=datagram))
                # Run tegrastats
                if datagram == "start":
                    self.tegra.open(interval=500)
                elif datagram == "stop":
                    self.tegra.close()
            except socket.timeout:
                #print("Timeout!")
                pass

    def close(self):
        print("End Server")
        self.sock_ctrl.close()
        os.remove(JtopServer.PIPE_JTOP_CTRL)
        self.socket_stats.close()
        os.remove(JtopServer.PIPE_JTOP_STATS)

    def tegra_stats(self, stats):
        print(stats)
        self.socket_stats.sendto("stats", JtopServer.PIPE_JTOP_STATS)
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from os import path
import curses
# Page class definition
from .jtopgui import Page
# Graphics elements
from .lib.common import (label_freq,
                         size_min)
from .lib.linear_gauge import linear_gauge, GaugeName
from .lib.chart import Chart
from .lib.button import Button


class MEM(Page):

    def __init__(self, stdscr, jetson, refresh):
        super(MEM, self).__init__("MEM", stdscr, jetson, refresh)
        # Initialize MEM chart
        self.chart_ram = Chart(jetson, "RAM", refresh, self.update_chart,
                               type_value=float,
                               color=curses.color_pair(6),
                               color_chart=[curses.color_pair(12)])
        # Initialize buttons
        self.button_cache = Button(stdscr, "c", action=self.action_cache)
        self.button_swap = Button(stdscr, "s", action=self.action_swap)
        self.button_increase = Button(stdscr, "+", action=self.action_increase, underline=False)
        self.button_decrease = Button(stdscr, "-", action=self.action_decrease, underline=False)

    def action_cache(self, key):
        if self.jetson.userid == 0:
            self.jetson.swap.clearCache()

    def action_swap(self, key):
        if self.jetson.userid == 0:
            # Change status swap
            self.jetson.swap.enable = not self.jetson.swap.enable

    def action_increase(self, key):
        if self.jetson.userid == 0:
            swap_enable = self.jetson.swap.enable
            # Enable nvpmodel control
            if not swap_enable:
                self.jetson.swap.increase()

    def action_decrease(self, key):
        if self.jetson.userid == 0:
            swap_enable = self.jetson.swap.enable
            # Enable nvpmodel control
            if not swap_enable:
                self.jetson.swap.decrease()

    def update_chart(self, jetson, name):
        parameter = jetson.stats.get("RAM", {})
        # Get max value if is present
        max_val = parameter.get("tot", 100)
        # Get unit
        unit = parameter.get("unit", "M")
        # Get value
        value = parameter.get("use", 0)
        info = size_min(max_val, start=unit)
        # Append in list
        return {
            'value': [value / info[1]],
            'max': info[0],
            'unit': info[2]
        }

    def swap_menu(self, lc, size, start, width):
        line_counter = lc + 1
        # SWAP linear gauge info
        swap_status = self.jetson.stats.get('SWAP', {})
        swap_cached = swap_status.get('cached', {})
        szw, divider, unit = size_min(swap_status.get('tot', 0), start=swap_status.get('unit', ''))
        percent = "{use}{unit}B/{tot}{unit}B".format(use=swap_status.get('use', 0) / divider, tot=szw, unit=unit)
        # Make label cache
        label = "(cached {size}{unit}B)".format(size=swap_cached.get('size', '0'), unit=swap_cached.get('unit', ''))
        self.stdscr.addstr(line_counter, start, "Swap", curses.A_BOLD)
        self.stdscr.addstr(line_counter, width - len(label) - 1, label, curses.A_NORMAL)
        # Add all swaps
        for swap in self.jetson.swap.swaps():
            line_counter += 1
            value = int(swap['used'] / float(swap['size']) * 100.0)
            # Extract size swap and unit
            szw, divider, unit = size_min(swap['size'])
            used = swap['used'] / divider
            # Change color for type partition
            if swap['type'] == 'partition':
                color = curses.color_pair(5)
            elif swap['type'] == 'file':
                color = curses.color_pair(3)
            else:
                color = curses.color_pair(6)
            linear_gauge(self.stdscr, offset=line_counter, size=size, start=start,
                         name=GaugeName(path.basename(swap['name']), color=color),
                         value=value,
                         percent="{use}/{tot}{unit}b".format(use=int(round(used)), tot=round(szw, 1), unit=unit),
                         label="P={prio: d}".format(prio=int(swap['prio'])))
        # Draw total swap gauge
        line_counter += 1
        self.stdscr.hline(line_counter, start, curses.ACS_HLINE, size - 1)
        line_counter += 1
        linear_gauge(self.stdscr, offset=line_counter, size=size, start=start,
                     name=GaugeName('TOT', color=curses.color_pair(6)),
                     value=int(swap_status.get('use', 0) / float(swap_status.get('tot', 1)) * 100.0),
                     percent=percent,
                     status='ON' if swap_status else 'OFF')

    def draw(self, key, mouse):
        # Screen size
        height, width, first = self.size_page()
        # Set size chart memory
        size_x = [1, width * 1 // 2 + 5]
        size_y = [first + 1, height * 1 // 2 - 1]
        # RAM linear gauge info
        line_counter = size_y[1] + 2
        # Read RAM status and make gaugues
        ram_status = self.jetson.stats['RAM']
        lfb_status = self.jetson.stats['RAM']['lfb']
        szw, divider, unit = size_min(ram_status['tot'], start=ram_status['unit'])
        # lfb label
        percent = "{use:2.1f}{unit}/{tot:2.1f}{unit}B".format(use=ram_status['use'] / divider, unit=unit, tot=szw)
        label_lfb = "(lfb {nblock}x{size}{unit}B)".format(nblock=lfb_status['nblock'],
                                                          size=lfb_status['size'],
                                                          unit=lfb_status['unit'])
        # Draw the GPU chart
        self.chart_ram.draw(self.stdscr, size_x, size_y, label="{percent} - {lfb}".format(percent=percent, lfb=label_lfb))
        # Make swap list file
        self.swap_menu(lc=first, start=size_x[1] + 3, size=size_x[1] - 13, width=width)
        # Draw the Memory gague
        linear_gauge(self.stdscr, offset=line_counter, size=width - 1,
                     start=1,
                     name=GaugeName('Mem', color=curses.color_pair(6)),
                     value=int(ram_status['use'] / float(ram_status['tot']) * 100.0),
                     label=label_lfb,
                     percent=percent)
        # IRAM linear gauge info
        if 'IRAM' in self.jetson.stats:
            iram_status = self.jetson.stats['IRAM']
            line_counter += 1
            szw, divider, unit = size_min(iram_status['tot'], start=iram_status['unit'])
            # lfb label
            percent = "{use:2.1f}{unit}/{tot:2.1f}{unit}B".format(use=iram_status['use'] / divider, unit=unit, tot=szw)
            label_lfb = "(lfb {size}{unit}B)".format(size=iram_status['lfb']['size'],
                                                     unit=iram_status['lfb']['unit'])
            linear_gauge(self.stdscr, offset=line_counter, size=width - 1,
                         start=1,
                         name=GaugeName('Imm', color=curses.color_pair(6)),
                         value=int(iram_status['use'] / float(iram_status['tot']) * 100.0),
                         label=label_lfb,
                         percent=percent)
        # EMC linear gauge info
        line_counter += 1
        emc = self.jetson.stats.get('EMC', {})
        linear_gauge(self.stdscr, offset=line_counter, size=width - 1,
                     start=1,
                     name=GaugeName('EMC', color=curses.color_pair(6)),
                     value=emc.get('val', 0),
                     status='ON' if emc else 'SUDO SUGGESTED',
                     label=label_freq(emc))
        if self.jetson.userid == 0:
            # Clear cache button
            self.button_cache.draw(first + height - 7, 1, key, mouse)
            clear_cache = "Clear cache"
            self.stdscr.addstr(first + height - 6, 7, clear_cache, curses.A_NORMAL)
        if self.jetson.userid == 0:
            # Swap controller
            self.button_swap.draw(first + height - 4, 1, key, mouse)
            self.stdscr.addstr(first + height - 4, 7, "Extra", curses.A_UNDERLINE)
            enable_swap = "Swap"
            self.stdscr.addstr(first + height - 3, 7, enable_swap, curses.A_NORMAL)
            # Status swap
            swap_enable = self.jetson.swap.enable
            self.stdscr.addstr(first + height - 4, 11 + len(enable_swap), "Status", curses.A_UNDERLINE)
            self.stdscr.addstr(first + height - 3, 11 + len(enable_swap),
                               "Enabled" if swap_enable else "Disable",
                               curses.A_BOLD if swap_enable else curses.A_NORMAL)
            # Swap boxes
            start_pos = 10 + len(enable_swap)
            if not swap_enable:
                # Draw keys to decrease size swap
                self.button_decrease.draw(first + height - 4, start_pos + 10, key, mouse)
                # Draw selected number
                swp_size = int(self.jetson.swap.size)
                if swp_size > len(self.jetson.swap):
                    color = curses.color_pair(2)
                elif swp_size < len(self.jetson.swap):
                    color = curses.color_pair(3)
                else:
                    color = curses.A_NORMAL
                self.stdscr.addstr(first + height - 3, start_pos + 16, "{size: <2}".format(size=swp_size), color)
                self.stdscr.addstr(first + height - 3, start_pos + 18, "Gb", curses.A_BOLD)
                # Draw keys to increase size swap
                self.button_increase.draw(first + height - 4, start_pos + 21, key, mouse)
            # else:
            #    # Print folder swapfile
            #    self.stdscr.addstr(first + height - 3, start_pos + 11, "{folder}".format(folder=self.jetson.swap.file), curses.A_BOLD)
# EOF



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2020 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

"""
    Graphic reference:
    - http://urwid.org/examples/index.html
    - https://npyscreen.readthedocs.io/
    - https://github.com/chubin/cheat.sh
    - https://stackoverflow.com/questions/6840420/python-rewrite-multiple-lines-in-the-console
    - https://docs.python.org/3.3/howto/curses.html#attributes-and-color
    - http://toilers.mines.edu/~jrosenth/101python/code/curses_plot/
"""
import os
import argparse
# Logging
import logging
# control command line
import curses
# Load colors
from .github import jetpack_missing, board_missing
# Tegrastats objext reader
from .jtop import jtop, get_version
# GUI jtop interface
from .gui import JTOPGUI, ALL, CPU, GPU, MEM, CTRL, INFO
# Create logger for jplotlib
logger = logging.getLogger(__name__)
# Reference repository
REPOSITORY = "https://github.com/rbonghi/jetson_stats/issues"
LOOP_SECONDS = 5


class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    @staticmethod
    def ok():
        return bcolors.OKGREEN + "OK" + bcolors.ENDC

    @staticmethod
    def warning():
        return bcolors.WARNING + "WARN" + bcolors.ENDC

    @staticmethod
    def fail():
        return bcolors.FAIL + "ERR" + bcolors.ENDC


def main():
    # Add arg parser
    parser = argparse.ArgumentParser(description='jtop is system monitoring utility and runs on terminal')
    parser.add_argument('--debug', dest="debug", help='Run with debug logger', action="store_true", default=False)
    parser.add_argument('--no-warnings', dest="no_warnings", help='Do not show warnings', action="store_true", default=False)
    parser.add_argument('--restore', dest="restore", help='Reset Jetson configuration', action="store_true", default=False)
    parser.add_argument('--loop', dest="loop", help='Automatically switch page every {sec}s'.format(sec=LOOP_SECONDS), action="store_true", default=False)
    parser.add_argument('-r', '--refresh', dest="refresh", help='refresh interval', type=int, default='500')
    parser.add_argument('-p', '--page', dest="page", help='Open fix page', type=int, default=1)
    parser.add_argument('-v', '--version', action='version', version='%(prog)s {version}'.format(version=get_version()))
    # Parse arguments
    args = parser.parse_args()
    # Set logging level
    if args.debug:
        logging.basicConfig(level=logging.DEBUG, filename='jtop.log', filemode='w',
                            format='%(name)s - %(levelname)s - %(message)s')
    else:
        logging.basicConfig()
    # Open tegrastats reader and run the curses wrapper
    try:
        with jtop(interval=args.refresh) as jetson:
            if not args.restore:
                try:
                    # Call the curses wrapper
                    curses.wrapper(JTOPGUI, args.refresh, jetson, [ALL, GPU, CPU, MEM, CTRL, INFO], init_page=args.page, loop=args.loop, seconds=LOOP_SECONDS)
                except KeyboardInterrupt as x:
                    # Catch keyboard interrupt and close
                    logger.info("Closed with CTRL-C [{status}]".format(status=x))
                except SystemExit as x:
                    # Catch keyboard interrupt and close
                    logger.info("System exit {status}".format(status=x))
            else:
                if jetson.userid == 0:
                    # If enable restore:
                    # * Disable jetson_clocks
                    if jetson.jetson_clocks:
                        jetson.jetson_clocks.start = False
                        print("[{status}] Stop jetson_clocks service".format(status=bcolors.ok()))
                        jetson.jetson_clocks.enable = False
                        print("[{status}] Disable jetson_clocks service".format(status=bcolors.ok()))
                    # * Set fan speed to 0
                    if jetson.fan:
                        jetson.fan.speed = 0
                        print("[{status}] Fan speed = 0".format(status=bcolors.ok()))
                        jetson.fan.control = True
                        print("[{status}] Fan temp_control = 1".format(status=bcolors.ok()))
                    # * Delete fan_configuration
                        clear = jetson.fan.clear()
                        status = bcolors.ok() if clear else bcolors.fail()
                        print("[{status}] Clear Fan Configuration".format(status=status))
                    # * Delete jetson_clocks configuration
                    if jetson.jetson_clocks:
                        clear = jetson.jetson_clocks.clear()
                        status = bcolors.ok() if clear else bcolors.fail()
                        print("[{status}] Clear Jetson Clock Configuration".format(status=status))
                else:
                    print("[{status}] Please run with sudo".format(status=bcolors.fail()))
        if not args.no_warnings:
            # Check if jetpack is missing
            if not os.environ["JETSON_TYPE"] and os.environ["JETSON_BOARD"] and "DEBUG" not in os.environ["JETSON_MACHINE"]:
                print("[{status}] {link}".format(status=bcolors.warning(), link=board_missing(REPOSITORY, get_version())))
            # Check if jetpack is missing
            if os.environ["JETSON_JETPACK"] == "UNKNOWN" and os.environ["JETSON_L4T"] != "N.N.N":
                print("[{status}] {link}".format(status=bcolors.warning(), link=jetpack_missing(REPOSITORY, get_version())))
    except jtop.JtopException as e:
        # Print error and close
        print("[{status}] {error}".format(status=bcolors.fail(), error=e.message))
        print("Run jetson_config (health page) to fix it")
        print("or open an issue on {url}".format(url=REPOSITORY))


if __name__ == "__main__":
    main()
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import curses
from .common import check_curses


class GaugeName:
    def __init__(self, text, color=curses.A_NORMAL):
        self.text = text
        self.color = color


class GaugeBar:
    def __init__(self, number, color):
        self.number = number
        self.color = color


@check_curses
def linear_gauge(stdscr, offset=0, start=0, size=10, name="", value=0, status="ON", percent="", label="", bar="|"):
    name = GaugeName(name) if isinstance(name, str) else name
    label = GaugeName(label) if isinstance(label, str) else label
    values = (GaugeBar(value, curses.color_pair(2)), ) if isinstance(value, int) else value
    # Evaluate size withuout short name
    name_size = len(name.text)
    size_bar = size - name_size - 4
    # Show short name linear gauge
    stdscr.addstr(offset, start, name.text, name.color)
    # Check if value is not a string
    if 'ON' in status:
        # Show bracket linear gauge and label and evaluate size withuout size labels and short name
        size_bar -= (len(label.text) + 1) if label.text else 0
        stdscr.addstr(offset, start + name_size + 1, "[" + " " * size_bar + "]", curses.A_BOLD)
        if label.text:
            stdscr.addstr(offset, start + name_size + 1 + size_bar + 3, label.text, label.color)
        # Show progress value linear gauge
        total = sum([val.number for val in values])
        n_bar = int(float(total) * float(size_bar) / 100.0)
        if n_bar >= 0:
            # Build progress barr string
            str_progress_bar = bar * n_bar
            percent_label = percent if percent else str(total) + "%"
            str_progress_bar = str_progress_bar[:size_bar - len(percent_label)] + percent_label
            # Split string in green and grey part
            counter = 0
            old_val = 0
            for value in values:
                val_bar = int(float(old_val + value.number) * float(size_bar) / 100.0)
                stdscr.addstr(offset, start + name_size + 2 + counter, str_progress_bar[counter:val_bar], value.color)
                counter += val_bar
                old_val += value.number
            # Draw grey part
            grey_part = str_progress_bar[n_bar:]
            stdscr.addstr(offset, start + name_size + 2 + size_bar - len(grey_part), grey_part, curses.A_DIM)
    else:
        # Show bracket linear gauge and label
        stdscr.addstr(offset, start + name_size + 1, ("[{value:>" + str(size_bar) + "}]").format(value=" "), curses.color_pair(7))
        # Show bracket linear gauge and label
        status = status if status else "OFF"
        stdscr.addstr(offset, start + name_size + 4, status, curses.color_pair(7))
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# Logging
import logging
# Launch command
import subprocess as sp
# Threading
from threading import Thread
# Tegrastats parser
from .tegra_parse import VALS, MTS, RAM, SWAP, IRAM, CPUS, TEMPS, WATTS
# Create logger for tegrastats
logger = logging.getLogger(__name__)


class Tegrastats:
    """
        - Subprocess read:
        https://stackoverflow.com/questions/375427/non-blocking-read-on-a-subprocess-pipe-in-python/4896288#4896288
        - Property
        https://www.programiz.com/python-programming/property
    """

    class TegrastatsException(Exception):
        pass

    def __init__(self, path):
        # Initialize jetson stats
        self._stats = {}
        # Start process tegrastats
        self.path = path
        # Define Tegrastats process
        self.p = None
        # Initialize observer
        self._observers = set()

    def _decode(self, text):
        # Find and parse all single values
        stats = VALS(text)
        # Parse if exist MTS
        mts = MTS(text)
        if mts:
            stats['MTS'] = mts
        # Parse RAM
        stats['RAM'] = RAM(text)
        # If exists parse SWAP
        swap = SWAP(text)
        if swap:
            stats['SWAP'] = swap
        # If exists parse IRAM
        iram = IRAM(text)
        if iram:
            stats['IRAM'] = iram
        # Parse CPU status
        stats['CPU'] = CPUS(text)
        # Parse temperatures
        stats['TEMP'] = TEMPS(text)
        # Parse Watts
        stats['WATT'] = WATTS(text)
        return stats

    def _read_tegrastats(self):
        try:
            while self.p.poll() is None:
                out = self.p.stdout
                if out is not None:
                    # Read line process output
                    line = out.readline()
                    # Decode line in UTF-8
                    tegrastats_data = line.decode("utf-8")
                    # Decode and store
                    self._stats = self._decode(tegrastats_data)
                    # Notifiy all observers
                    for observer in self._observers:
                        observer(self._stats)
        except SystemExit:
            logger.error("System exit", exc_info=True)
        except AttributeError:
            logger.error("Attribute error", exc_info=True)

    def attach(self, observer):
        self._observers.add(observer)

    def detach(self, observer):
        self._observers.discard(observer)

    def open(self, interval=500):
        try:
            # Launch subprocess or raise and exception
            self.p = sp.Popen([self.path, '--interval', str(interval)], stdout=sp.PIPE)
            # Start thread Service client
            self._thread = Thread(target=self._read_tegrastats, args=[])
            self._thread.setDaemon = True
            self._thread.start()
            return True
        except OSError:
            logger.error("Tegrastats not in list!")
            raise Tegrastats.TegrastatsException("Tegrastats is not available on this hardware")
        return False

    def close(self):
        if self.p is not None:
            try:
                self.p.kill()
                return True
            except OSError:
                pass
        return False
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2020 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

# Logging
import logging
# Launch command
import subprocess as sp
# Create logger for jplotlib
logger = logging.getLogger(__name__)

SWAP_MAX_SIZE = 15
SWAP_MIN_SIZE = 2


class Swap(object):

    class SwapException(Exception):
        pass

    def __init__(self, dir_swap="", default=8, swap_name="swfile"):
        # Set default folder swap
        self.dir = dir_swap
        self.swap_name = swap_name
        self.swap_info = {}
        # Define actual size and new size variable
        self.actual_size = 0
        self.new_size = default
        # Initialize auto mount
        self.auto = True
        # Check if exist jetson_swap
        exist = sp.call('command -v jetson_swap >> /dev/null', shell=True)
        if exist != 0:
            raise Swap.SwapException("jetson_swap does not exist!")
        # Load swap information
        self.update()

    def update(self):
        swap_status = sp.Popen(['jetson_swap', '--status', '--dir', self.dir, '--name', self.swap_name], stdout=sp.PIPE, stderr=sp.PIPE)
        out, _ = swap_status.communicate()
        if out:
            swap_data = out.decode("utf-8")
            swap_data = swap_data.split("\t")
            # Load swap informations
            self.swap_info['file'] = swap_data[0].strip()
            self.swap_info['type'] = swap_data[1].strip()
            self.swap_info['size'] = int(swap_data[2].strip()) / 1000000.0
            self.swap_info['used'] = int(swap_data[3].strip()) / 1000.0
            self.swap_info['priority'] = int(swap_data[4].strip())
            # Update size
            self.actual_size = int(self.swap_info['size'])
        else:
            self.swap_info = {}

    def swaps(self):
        swap_status = sp.Popen(['swapon', '--show', '--raw', '--byte'], stdout=sp.PIPE, stderr=sp.PIPE)
        out, _ = swap_status.communicate()
        swaps = []
        if out:
            swap_data = out.decode("utf-8")
            # Read all data
            names = []
            for line in swap_data.split("\n"):
                # Extract names
                # The names are: name, type, size, used, prio
                if not names:
                    names = line.split()
                    continue
                # Extract data
                datas = line.split()
                if datas:
                    swaps += [{name.lower(): int(data) if data.isdigit() else data for name, data in zip(names, datas)}]
        return swaps

    def increase(self):
        if self.size + 1 <= SWAP_MAX_SIZE:
            self.size += 1
            return True
        else:
            return False

    def decrease(self):
        if self.size - 1 >= SWAP_MIN_SIZE:
            self.size -= 1
            return True
        else:
            return False

    @property
    def file(self):
        return self.dir + "/" + self.swap_name

    @property
    def size(self):
        return self.new_size

    @size.setter
    def size(self, val):
        self.new_size = val

    def __len__(self):
        return self.actual_size

    def clearCache(self):
        """
        Clear cache following https://coderwall.com/p/ef1gcw/managing-ram-and-swap
        """
        clear_cache = sp.Popen(['sysctl', 'vm.drop_caches=3'], stdout=sp.PIPE, stderr=sp.PIPE)
        out, _ = clear_cache.communicate()
        return True if out else False

    @property
    def enable(self):
        self.update()
        return True if self.swap_info else False

    @enable.setter
    def enable(self, value):
        if not isinstance(value, bool):
            raise Exception("Use a boolean")
        # Enable or disable swap
        if value:
            self._enable()
        else:
            self._disable()

    def _enable(self):
        # List swap command
        swap_cmd = ['jetson_swap', '--size', str(self.new_size), '--dir', self.dir, '--name', self.swap_name]
        # Add auto command if request
        if self.auto:
            swap_cmd += ['--auto']
        # Run script
        sp.Popen(swap_cmd, stdout=sp.PIPE, stderr=sp.PIPE)

    def _disable(self):
        # List swap command
        swap_cmd = ['jetson_swap', '--off', '--dir', self.dir, '--name', self.swap_name]
        # Run script
        sp.Popen(swap_cmd, stdout=sp.PIPE, stderr=sp.PIPE)
        # Remove swapfile if exist
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import curses
from curses.textpad import rectangle
from .jtopgui import Page
# Graphics elements
from .lib.common import (size_min,
                         label_freq)
from .lib.linear_gauge import linear_gauge, GaugeName, GaugeBar
# Menu GUI pages
from .jtopguimenu import (plot_watts,
                          compact_info,
                          plot_temperatures,
                          plot_CPUs)
# Actual variable printed
# LIST_PRINT = ['CPU', 'MTS', 'RAM', 'IRAM', 'SWAP', 'EMC', 'GR3D', 'TEMP', 'WATT', 'FAN', 'APE', 'NVENC', 'NVDEC', 'MSENC']


class ALL(Page):

    def __init__(self, stdscr, jetson, refresh):
        super(ALL, self).__init__("ALL", stdscr, jetson, refresh)

    def draw(self, key, mouse):
        """
            Update screen with values
        """
        # Screen size
        height, width, first = self.size_page()
        line_counter = first + 1
        # Plot Status CPU
        line_counter = plot_CPUs(self.stdscr, line_counter, self.jetson.stats['CPU'], width)
        # Plot MTS
        if 'MTS' in self.jetson.stats:
            line_counter += 1
            self.stdscr.addstr(line_counter, 0, "MTS ", curses.color_pair(5))
            # Show FG linear gauge
            linear_gauge(self.stdscr, offset=line_counter, start=4, size=width // 2 - 2,
                         name=GaugeName('FG', color=curses.color_pair(5)),
                         value=self.jetson.stats['MTS']['fg'])
            # Show BG linear gauge
            linear_gauge(self.stdscr, offset=line_counter, start=2 + width // 2, size=width // 2 - 2,
                         name=GaugeName('BG', color=curses.color_pair(5)),
                         value=self.jetson.stats['MTS']['bg'])
        # RAM linear gauge info
        line_counter += 1
        ram_status = self.jetson.stats['RAM']
        lfb_status = self.jetson.stats['RAM']['lfb']
        szw, divider, unit = size_min(ram_status['tot'], start=ram_status['unit'])
        # lfb label
        percent = "{use:2.1f}{unit}/{tot:2.1f}{unit}B".format(use=ram_status['use'] / divider, unit=unit, tot=szw)
        label_lfb = "(lfb {nblock}x{size}{unit}B)".format(nblock=lfb_status['nblock'],
                                                          size=lfb_status['size'],
                                                          unit=lfb_status['unit'])
        # Plot Linear Gauge
        ram_bar = GaugeBar(int(ram_status['use'] / float(ram_status['tot']) * 100.0), curses.color_pair(6))
        linear_gauge(self.stdscr, offset=line_counter, size=width,
                     name=GaugeName('Mem', color=curses.color_pair(6)),
                     value=(ram_bar, ),
                     label=label_lfb,
                     percent=percent)
        # IRAM linear gauge info
        if 'IRAM' in self.jetson.stats:
            iram_status = self.jetson.stats['IRAM']
            line_counter += 1
            szw, divider, unit = size_min(iram_status['tot'], start=iram_status['unit'])
            # lfb label
            percent = "{use:2.1f}{unit}/{tot:2.1f}{unit}B".format(use=iram_status['use'] / divider, unit=unit, tot=szw)
            label_lfb = "(lfb {size}{unit}B)".format(size=iram_status['lfb']['size'],
                                                     unit=iram_status['lfb']['unit'])
            linear_gauge(self.stdscr, offset=line_counter, size=width,
                         name=GaugeName('Imm', color=curses.color_pair(6)),
                         value=int(iram_status['use'] / float(iram_status['tot']) * 100.0),
                         label=label_lfb,
                         percent=percent)
        # SWAP linear gauge info
        line_counter += 1
        swap_status = self.jetson.stats.get('SWAP', {})
        swap_cached = swap_status.get('cached', {})
        # lfb label
        szw, divider, unit = size_min(swap_status.get('tot', 0), start=swap_status.get('unit', ''))
        percent = "{use}{unit}B/{tot}{unit}B".format(use=swap_status.get('use', 0) / divider, tot=szw, unit=unit)
        label_lfb = "(cached {size}{unit}B)".format(size=swap_cached.get('size', '0'),
                                                    unit=swap_cached.get('unit', ''))
        linear_gauge(self.stdscr, offset=line_counter, size=width,
                     name=GaugeName('Swp', color=curses.color_pair(6)),
                     value=int(swap_status.get('use', 0) / float(swap_status.get('tot', 1)) * 100.0),
                     label=label_lfb,
                     percent=percent,
                     status='ON' if swap_status else 'OFF')
        # EMC linear gauge info
        line_counter += 1
        emc = self.jetson.stats.get('EMC', {})
        linear_gauge(self.stdscr, offset=line_counter, size=width,
                     name=GaugeName('EMC', color=curses.color_pair(6)),
                     value=emc.get('val', 0),
                     status='ON' if emc else 'SUDO SUGGESTED',
                     label=label_freq(emc))
        # GPU linear gauge info
        line_counter += 2
        gpu = self.jetson.stats.get('GR3D', {})
        linear_gauge(self.stdscr, offset=line_counter, size=width,
                     name=GaugeName('GPU', color=curses.color_pair(6)),
                     value=gpu.get('val', 0),
                     label=label_freq(gpu),
                     status='ON' if gpu else 'SUDO SUGGESTED')
        # Status disk
        line_counter += 1
        disk_status = self.jetson.disk
        linear_gauge(self.stdscr, offset=line_counter, size=width,
                     name=GaugeName('Dsk', color=curses.color_pair(3)),
                     value=int(float(disk_status['used']) / float(disk_status['total']) * 100.0),
                     percent="{0:2.1f}GB/{1:2.1f}GB".format(disk_status['used'], disk_status['total']),
                     bar="#")
        # Last part of information
        mini_menu = 1
        mini_menu += 1 if self.jetson.stats['TEMP'] else 0
        mini_menu += 1 if self.jetson.stats['WATT'] else 0
        column_width = (width) // (mini_menu)
        column_height = height - line_counter - 3 + first
        # Make rectangle
        rectangle(self.stdscr, line_counter + 1, 0, line_counter + 1 + column_height, width - 1)
        # Plot compact info
        compact_info(self.stdscr, 0, line_counter + 1, column_width + 2, column_height, self.jetson)
        # Plot temperatures
        if self.jetson.stats['TEMP']:
            self.add_line(line_counter + 1, column_width + 2, column_height)
            plot_temperatures(self.stdscr, column_width + 2, line_counter + 1, column_width - 4, column_height, self.jetson)
        # plot watts
        if self.jetson.stats['WATT']:
            self.add_line(line_counter + 1, 2 * column_width - 2, column_height)
            plot_watts(self.stdscr, 2 * column_width - 1, line_counter + 1, column_width + 2, column_height, self.jetson)

    def add_line(self, pos_y, pos_x, height):
        """
        http://www.melvilletheatre.com/articles/ncurses-extended-characters/index.html
        """
        self.stdscr.addch(pos_y, pos_x, curses.ACS_TTEE)
        self.stdscr.vline(pos_y + 1, pos_x, curses.ACS_VLINE, height - 1)
        self.stdscr.addch(pos_y + height, pos_x, curses.ACS_BTEE)
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
"""
JTOP is a complete controller of all systems in your NVIDIA Jetson
 * Tegrastats
 * NVP Model
 * Fan
 * Status board (i.g. Model version, Jetpack, ... )

You can initialize the jtop node like a file i.g.

.. code-block:: python

    with jtop() as jetson:
        stat = jetson.stats

Or manually start up with the basic function `open/close`

.. code-block:: python

    jetson = jtop()
    jetson.open()
    stat = jetson.stats
    jetson.close()

Jtop include all informations about your board. The default properties are:
 * stats
 * nvpmodel
 * fan
 * board
 * disk

Follow the next attributes to know in detail how it works.
"""
import re
import os
import sys
# Logging
import logging

from .core import NVPmodel
from .core import Tegrastats
from .core import Fan
from .core import JetsonClocks
from .core import Swap
from .core import cpuinfo
from .core import nvjpg
from .core import (import_os_variables,
                   get_uptime,
                   status_disk,
                   get_local_interfaces,
                   StatusObserver)

# Create logger for jplotlib
logger = logging.getLogger(__name__)
# Version match
VERSION_RE = re.compile(r""".*__version__ = ["'](.*?)['"]""", re.S)


def import_jetson_variables():
    JTOP_FOLDER, _ = os.path.split(__file__)
    return import_os_variables(JTOP_FOLDER + "/jetson_variables", "JETSON_")


def get_version():
    """
    Show the version of this package

    :return: Version number
    :rtype: string
    """
    # Load version package
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, "__init__.py")) as fp:
        VERSION = VERSION_RE.match(fp.read()).group(1)
    return VERSION


class jtop(StatusObserver):
    """
    with this class you can control your jtop statistics and manage your board

    :param interval: Interval update tegrastats and other statistic function
    :type interval: int
    """
    class JtopException(Exception):
        """ Jtop general exception """
        pass

    def __init__(self, interval=500):
        # Load configuration file path
        config_file = sys.prefix + "/local/jetson_stats"
        # Version package
        self.version = get_version()
        # Initialize observer
        self._observers = set()
        self._started = False
        # Load all Jetson variables
        logger.info("Load jetson variables from script")
        for k, v in import_jetson_variables().items():
            logger.debug("New Enviroment variable {}:{}".format(k, v))
            os.environ[k] = v
        # Initialize jetson_clocks controller
        try:
            self.jc = JetsonClocks(config_file)
        except JetsonClocks.JCException as e:
            raise jtop.JtopException(e)
        # Initialize NVP model
        try:
            self.nvp = NVPmodel(jetson_clocks=self.jc)
        except NVPmodel.NVPmodelException:
            self.nvp = None
        # Find all fans availables
        self.qfan = None
        LIST_FANS = [('/sys/kernel/debug/tegra_fan/', False), ('/sys/devices/pwm-fan/', True)]
        for path, temp_control in LIST_FANS:
            try:
                self.qfan = Fan(path, self.jc, config_file, temp_control=temp_control)
                logger.info("Fan {} loaded!".format(path))
                break
            except Fan.FanException:
                logger.info("Fan {} not loaded".format(path))
        # Start process tegrastats
        tegrastats_file = ""
        for f_tegra in ['/usr/bin/tegrastats', '/home/nvidia/tegrastats']:
            if os.path.isfile(f_tegra):
                logger.info("Load tegrastats {}".format(f_tegra))
                tegrastats_file = f_tegra
                break
        if not tegrastats_file:
            raise jtop.JtopException("Tegrastats is not availabe on this board")
        try:
            # Initialize Swap controller
            self.swap = Swap()
        except Swap.SwapException as e:
            raise jtop.JtopException(e)
        # Initialize Tegrastats controller
        self._stats = {}
        self.tegrastats = Tegrastats(tegrastats_file, interval)

    @property
    def userid(self):
        """ Linux User ID """
        return os.getuid()

    @property
    def architecture(self):
        """ CPU architecture """
        return cpuinfo.info()

    @property
    def fan(self):
        """ Fan object controller """
        return self.qfan

    @property
    def disk(self):
        """ Disk status properties """
        return status_disk()

    @property
    def jetson_clocks(self):
        """ JetsonClock controller """
        return self.jc

    @property
    def uptime(self):
        """ Up time """
        return get_uptime()

    @property
    def nvpmodel(self):
        """
        NVPmodel is the controller of your NVP model.
        From this object you read and set the status of your NVIDIA Jetson.
        """
        return self.nvp

    @property
    def nvjpg(self):
        return nvjpg()

    @property
    def local_interfaces(self):
        """ Local interfaces information """
        return get_local_interfaces()

    @property
    def board(self):
        """
        Detailed information of your board, with a complete list of:
         * Jetpack
         * L4T
         * Serial Number
         * ...
        """
        info = {"Machine": os.environ["JETSON_MACHINE"],
                "Jetpack": os.environ["JETSON_JETPACK"] + " [L4T " + os.environ["JETSON_L4T"] + "]"}
        board = {"TYPE": os.environ["JETSON_TYPE"],
                 "CODENAME": os.environ["JETSON_CODENAME"],
                 "SOC": os.environ["JETSON_SOC"],
                 "CHIP_ID": os.environ["JETSON_CHIP_ID"],
                 "BOARDIDS": os.environ["JETSON_BOARDIDS"],
                 "MODULE": os.environ["JETSON_MODULE"],
                 "BOARD": os.environ["JETSON_BOARD"],
                 "CUDA_ARCH_BIN": os.environ["JETSON_CUDA_ARCH_BIN"],
                 "SERIAL_NUMBER": os.environ["JETSON_SERIAL_NUMBER"].upper()}
        libraries = {"CUDA": os.environ["JETSON_CUDA"],
                     "cuDNN": os.environ["JETSON_CUDNN"],
                     "TensorRT": os.environ["JETSON_TENSORRT"],
                     "VisionWorks": os.environ["JETSON_VISIONWORKS"],
                     "OpenCV": os.environ["JETSON_OPENCV"],
                     "OpenCV-Cuda": os.environ["JETSON_OPENCV_CUDA"],
                     "VPI": os.environ["JETSON_VPI"],
                     "Vulkan": os.environ["JETSON_VULKAN_INFO"]}
        return {"info": info, "board": board, "libraries": libraries}

    @property
    def stats(self):
        """
        Detailed information of your board, with a complete list of:
         * Jetpack
         * L4T
         * Serial Number
         * ...
        """
        # Wait the deque not empty
        while not self._stats:
            pass
        # Return dictionary parsed
        return self._stats

    def open(self):
        """ Open tegrastats app and read all stats """
        if not self._started:
            try:
                self.tegrastats.open(self)
                self._started = True
            except Tegrastats.TegrastatsException as e:
                raise jtop.JtopException(e)

    def close(self):
        """ Close tegrastats app """
        self.tegrastats.close()
        self._started = False

    def attach(self, observer):
        """
        Attach an obserber to read the status of jtop

        :param observer: The function to call
        :type observer: function
        """
        self._observers.add(observer)
        # Autostart the jtop if is off
        if self._observers:
            self.open()

    def detach(self, observer):
        """
        Detach an obserber from jtop

        :param observer:  The function to detach
        :type observer: function
        """
        self._observers.discard(observer)

    def update(self, stats):
        """
        Update the status of jtop passing stats

        :param stats:  Statistic dictionary
        :type stats: dict
        """
        # Update nvpmodel
        if self.nvp is not None:
            self.nvp.update()
        # Update status from fan
        if self.qfan is not None:
            self.qfan.update()
            # Add fan status
            stats["FAN"] = self.qfan.status
        # Update status
        self._stats = stats
        # Notifiy all observers
        for observer in self._observers:
            if callable(observer):
                observer(self.stats)
            else:
                observer.update(self)

    def __enter__(self):
        """ Enter function for 'with' statement """
        self.open()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """ Exit function for 'with' statement """
        self.close()
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import curses
from .jtopgui import Page
# Graphics elements
from .lib.common import check_curses
# Graphic library
from .lib.chart import Chart
from .lib.button import Button, ButtonList
# Jetson Clock event
from ..core.jetson_clocks import JetsonClocks


class CTRL(Page):

    def __init__(self, stdscr, jetson, refresh):
        super(CTRL, self).__init__("CTRL", stdscr, jetson, refresh)
        # Only if exist a fan will be load a chart
        # Initialize FAN chart
        self.chart_fan = Chart(jetson, "FAN", refresh, self.update_chart, line="o", color=curses.color_pair(4), color_chart=[curses.color_pair(10)])
        if 'FAN' not in self.jetson.stats:
            self.chart_fan.statusChart(False, "NO FAN")
        # Initialize buttons
        self.service_start = Button(stdscr, key="s", action=self.action_service_start)
        self.service_enable = Button(stdscr, key="e", action=self.action_service_enable)
        # NVP Model controller
        self.nvp_increase = Button(stdscr, key="+", action=self.action_nvp_increase, underline=False)
        self.nvp_decrease = Button(stdscr, key="-", action=self.action_nvp_decrease, underline=False)
        mode_names = [mode["Name"] for mode in self.jetson.nvpmodel.modes]
        self.nvp_list = ButtonList(stdscr, range(len(mode_names)), mode_names, action=self.action_nvp)
        # Fan controller
        self.fan_status = Button(stdscr, key="f", action=self.action_fan_status)
        self.fan_status_increase = Button(stdscr, key="p", action=self.action_fan_increase)
        self.fan_status_decrease = Button(stdscr, key="m", action=self.action_fan_decrease)

    def action_nvp_increase(self, key):
        if self.jetson.userid == 0:
            # NVPmodel controller
            nvpmodel = self.jetson.nvpmodel
            if nvpmodel is not None:
                nvpmodel.increase()

    def action_nvp_decrease(self, key):
        if self.jetson.userid == 0:
            # NVPmodel controller
            nvpmodel = self.jetson.nvpmodel
            if nvpmodel is not None:
                nvpmodel.decrease()

    def action_nvp(self, key):
        if self.jetson.userid == 0:
            # Run nvpmodel on number selected
            nvpmodel = self.jetson.nvpmodel
            if nvpmodel is not None:
                # Read number
                number = int(key)
                # Set new nvpmodel
                nvpmodel.set(number)

    def action_service_enable(self, key):
        if self.jetson.userid == 0:
            # Start jetson_clocks
            self.jetson.jetson_clocks.enable = not self.jetson.jetson_clocks.enable

    def action_service_start(self, key):
        if self.jetson.userid == 0:
            # Start jetson_clocks
            self.jetson.jetson_clocks.start = not self.jetson.jetson_clocks.start

    def action_fan_status(self, key):
        if self.jetson.userid == 0:
            # FAN controller
            fan = self.jetson.fan
            if fan is not None:
                # Go to next configuration
                fan.conf_next()
                # Store configuration
                fan.store()

    def action_fan_increase(self, key):
        if self.jetson.userid == 0:
            # FAN controller
            fan = self.jetson.fan
            if fan is not None:
                fan.increase()
                # Store configuration
                fan.store()

    def action_fan_decrease(self, key):
        if self.jetson.userid == 0:
            # FAN controller
            fan = self.jetson.fan
            if fan is not None:
                fan.decrease()
                # Store configuration
                fan.store()

    def update_chart(self, jetson, name):
        parameter = jetson.stats.get("FAN", {})
        value = 'cpwm' if 'cpwm' in parameter else 'tpwm'
        # Get max value if is present
        max_val = parameter.get("max_val", 100)
        # Get unit
        unit = parameter.get("unit", "%")
        # Append in list
        return {
            'value': [parameter.get(value, 0)],
            'max': max_val,
            'unit': unit,
            'active': 'FAN' in jetson.stats
        }

    @check_curses
    def draw(self, key, mouse):
        """ Control board, check status jetson_clocks and change NVP model """
        # Screen size
        height, width, first = self.size_page()
        # Position information
        start_y = first + 1
        start_x = 1
        # Jetson Clocks status
        jc_field = "jetson_clocks"
        # Show status jetson_clocks
        self.stdscr.addstr(start_y + 1, start_x + 1, jc_field, curses.A_BOLD)
        try:
            status = self.jetson.jetson_clocks.status
            color = curses.color_pair(2) if status else curses.A_NORMAL
            jc_status_name = "Running" if status else "Stopped"
        except JetsonClocks.JCException:
            status = False
            # Fix error color
            color = curses.color_pair(11)
            jc_status_name = "SUDO SUGGESTED"
        self.stdscr.addstr(start_y + 1, start_x + len(jc_field) + 2, jc_status_name, color)
        # Show service status
        service = self.jetson.jetson_clocks.service
        jc_manual = status and service != "active"
        if self.jetson.userid == 0 and not jc_manual:
            # button start/stop jetson clocks
            self.service_start.draw(start_y + 2, start_x, key, mouse)
        # Field service
        service_string = "service"
        self.stdscr.addstr(start_y + 3, start_x + 5, service_string, curses.A_UNDERLINE)
        self.stdscr.addstr(start_y + 6, start_x + 5, service_string, curses.A_UNDERLINE)
        # Read status jetson_clocks
        if service == "active":
            color = curses.A_BOLD         # Running (Bold)
        elif service == "inactive":
            color = curses.A_NORMAL       # Normal (Grey)
        elif "ing" in service:
            color = curses.color_pair(3)  # Warning (Yellow)
        else:
            color = curses.color_pair(1)  # Error (Red)
        # Status service
        self.stdscr.addstr(start_y + 3, start_x + len(service_string) + 6,
                           service.capitalize(),
                           color)
        # button start/stop jetson clocks
        if self.jetson.userid == 0 and not jc_manual:
            self.service_enable.draw(start_y + 5, start_x, key, mouse)
        # Read status jetson_clocks
        enabled = self.jetson.jetson_clocks.enable
        self.stdscr.addstr(start_y + 6, start_x + len(service_string) + 6,
                           "Enable" if enabled else "Disable",
                           curses.A_BOLD if enabled else curses.A_NORMAL)
        # Build NVP model list
        nvpmodel = self.jetson.nvpmodel
        if nvpmodel is not None:
            self.stdscr.addstr(start_y + 9, start_x, "NVP model", curses.A_BOLD)
            if self.jetson.userid == 0:
                # Draw keys to decrease nvpmodel
                self.nvp_decrease.draw(start_y + 8, start_x + 10, key, mouse)
                # Draw selected number
                self.stdscr.addstr(start_y + 9, start_x + 16, str(nvpmodel.selected), curses.A_NORMAL)
                # Draw keys to increase nvpmodel
                self.nvp_increase.draw(start_y + 8, start_x + 18, key, mouse)
            # Write list of available modes
            self.nvp_list.draw(start_y + 11, start_x, width // 2 - start_x, key, mouse,
                               lstatus=[mode["status"] for mode in nvpmodel.modes],
                               select=nvpmodel.num)
        # Evaluate size chart
        size_x = [start_x + width // 2 + 1, width - start_x - 1]
        if self.jetson.userid == 0 and 'FAN' in self.jetson.stats:
            size_y = [start_y + 3, height - 3]
        else:
            size_y = [start_y, height - 3]
        # Add label
        label = ""
        if 'FAN' in self.jetson.stats:
            fan = self.jetson.stats['FAN']
            # Read status control fan
            if 'ctrl' in fan:
                ctrl_stat = "CTRL=" + ("Enable" if fan['ctrl'] else "Disable")
            else:
                ctrl_stat = ""
            # Add label
            if 'cpwm' in fan:
                label = "{current: >3}% of {target: >3}% {ctrl}".format(current=fan.get("cpwm", 0), target=fan.get("tpwm", 0), ctrl=ctrl_stat)
            else:
                label = "Target: {target: >3}% {ctrl}".format(target=fan.get("tpwm", 0), ctrl=ctrl_stat)
        # Draw the GPU chart
        self.chart_fan.draw(self.stdscr, size_x, size_y, label=label)
        # Add plot fan status
        if self.jetson.userid == 0 and 'FAN' in self.jetson.stats:
            # Mode
            if 'ctrl' in fan:
                self.fan_status.draw(start_y, start_x + width // 2, key, mouse)
                self.stdscr.addstr(start_y + 1, start_x + width // 2 + 6, self.jetson.fan.config.capitalize(), curses.A_BOLD)
            # Show speed buttons only if is in manual
            if self.jetson.fan.conf == 'manual':
                blk = start_x + width // 2 + 13
                # Draw keys to decrease fan speed
                self.fan_status_decrease.draw(start_y, blk, key, mouse)
                # Draw selected number
                self.stdscr.addstr(start_y, blk + 6, "Speed", curses.A_BOLD)
                speed_str = "{speed: 3}%".format(speed=self.jetson.fan.speed)
                self.stdscr.addstr(start_y + 1, blk + 6, speed_str, curses.A_NORMAL)
                # Draw keys to increase fan speed
                self.fan_status_increase.draw(start_y, blk + 13, key, mouse)
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.


import re
REGEXP = re.compile(r'(.+?): ((.*))')


class cpuinfo():
    """
    Find in cpuinfo information about the board
    """
    @staticmethod
    def info():
        cpus = {}
        with open("/proc/cpuinfo", "r") as fp:
            for line in fp:
                # Search line
                match = REGEXP.search(line)
                if match:
                    key = match.group(1).rstrip()
                    value = match.group(2).rstrip()
                    # Load value or if it is a new processor initialize a new field
                    if key == "processor":
                        n_proc = int(value)
                        cpus[n_proc] = {}
                    else:
                        # Load cpu info
                        cpus[n_proc][key] = value
        return cpus

    @staticmethod
    def models():
        # Load cpuinfo
        cpus = cpuinfo.info()
        models = {}
        # Find all models
        for cpu in cpus.values():
            model = cpu.get("model name", "")
            if model not in models:
                models[model] = 1
            else:
                models[model] += 1
        return models
# EOF



# -*- coding: UTF-8 -*-
# This file is part of the jetson_stats package (https://github.com/rbonghi/jetson_stats or http://rnext.it).
# Copyright (c) 2019 Raffaello Bonghi.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

import os


def nvjpg(path="/sys/kernel/debug/clk/nvjpg"):
    # Read status enable
    nvjpg = {}
    # Check if access to this file
    if os.access(path + "/clk_enable_count", os.R_OK):
        with open(path + "/clk_enable_count", 'r') as f:
            # Write status engine
            nvjpg['status'] = int(f.read()) == 1
    # Check if access to this file
    if os.access(path + "/clk_rate", os.R_OK):
        with open(path + "/clk_rate", 'r') as f:
            # Write status engine
            nvjpg['rate'] = int(f.read())
    return nvjpg
# EOF
